from __future__ import annotations

import pathlib
import tomllib
import uuid
from argparse import ArgumentParser
from enum import Enum
from typing import Any, Callable

import pandas as pd
import sqlalchemy
from googleapiutils2 import Sheets, SheetsValueRange, get_oauth2_creds  # type: ignore
from loguru import logger
from sqlalchemy import Column, DateTime, Integer, MetaData, Table, Text, func
from sqlalchemy.orm import declarative_base

from qualtrics_utils.codebook.generate import generate_codebook
from qualtrics_utils.misc import ExportedFile, T
from qualtrics_utils.survey import Surveys
from qualtrics_utils.utils import (
    coalesce_multiselect,
    create_mysql_engine,
    generate_sql_schema,
    parse_file_id,
    rename_columns,
)


class SyncType(Enum):
    SHEETS = "sheets"
    MYSQL = "mysql"


Base = declarative_base()


def get_status_table(table_name: str):
    return Table(
        table_name,
        Base.metadata,
        Column("id", Integer, primary_key=True, autoincrement=True),
        Column("timestamp", DateTime, server_default=func.now()),
        Column("last_response_id", Text),
        Column("continuation_token", Text),
        Column("file_id", Text),
    )


def format_name(survey_id: str, table_name: str | None, suffix: str):
    return table_name if table_name is not None else f"{survey_id}_{suffix}"


def format_status_name(survey_id: str, table_name: str | None):
    return format_name(survey_id=survey_id, table_name=table_name, suffix="status")


def format_responses_name(survey_id: str, table_name: str | None):
    return format_name(survey_id=survey_id, table_name=table_name, suffix="responses")


def format_status_row(exported_file: ExportedFile[T]):
    return dict(
        file_id=exported_file.file_id,
        timestamp=exported_file.timestamp.isoformat(),
        last_response_id=exported_file.last_response_id,
        continuation_token=exported_file.continuation_token,
    )


def setup_sql(
    responses_table_name: str | None,
    status_table_name: str | None,
    conn: sqlalchemy.Connection,
    restart: bool = False,
):
    def inner(exported_file: ExportedFile[pd.DataFrame]):
        survey_id = exported_file.survey_id
        df = exported_file.data

        metadata = MetaData()

        t_responses_table_name = format_responses_name(
            survey_id=survey_id, table_name=responses_table_name
        )
        t_status_table_name = format_status_name(
            survey_id=survey_id, table_name=status_table_name
        )

        responses_table = sqlalchemy.Table(t_responses_table_name, metadata)
        status_table = sqlalchemy.Table(t_status_table_name, metadata)

        if restart:
            responses_table.drop(conn, checkfirst=True)
            status_table.drop(conn, checkfirst=True)

        if not conn.dialect.has_table(conn, t_responses_table_name):
            responses_table = generate_sql_schema(
                df=df, table_name=t_responses_table_name, index_as_pk=True
            )
            responses_table.create(conn)

        if not conn.dialect.has_table(conn, t_status_table_name):
            status_table = get_status_table(table_name=t_status_table_name)
            status_table.create(conn)

        conn.commit()

    return inner


def get_last_status_sql(table_name: str | None, conn: sqlalchemy.Connection):
    def inner(survey_id: str):
        metadata = MetaData()

        status_table_name = format_status_name(
            survey_id=survey_id, table_name=table_name
        )

        status_table = Table(status_table_name, metadata, autoload_with=conn)

        query = status_table.select().order_by(status_table.c.id.desc()).limit(1)

        if (row := conn.execute(query).fetchone()) is not None:
            return row._asdict()

    return inner


def write_status_sql(
    table_name: str | None,
    conn: sqlalchemy.Connection,
):
    def inner(
        exported_file: ExportedFile[T],
    ):
        survey_id = exported_file.survey_id
        metadata = MetaData()

        status_table_name = format_status_name(
            survey_id=survey_id, table_name=table_name
        )
        status_table = Table(status_table_name, metadata, autoload_with=conn)

        conn.execute(
            status_table.insert().values(
                **format_status_row(exported_file),
            )
        )
        conn.commit()

    return inner


def write_responses_sql(
    table_name: str | None,
    conn: sqlalchemy.Connection,
):
    def inner(
        exported_file: ExportedFile[pd.DataFrame],
    ):
        survey_id = exported_file.survey_id
        df = exported_file.data

        metadata = MetaData()

        respones_table_name = format_responses_name(
            survey_id=survey_id, table_name=table_name
        )

        responses_table = Table(respones_table_name, metadata, autoload_with=conn)

        existing_columns = [col.name for col in responses_table.columns]
        df.drop(
            columns=[col for col in df.columns if col not in existing_columns],
            inplace=True,
        )

        df.to_sql(
            respones_table_name,
            conn,
            if_exists="append",
            index=True,
            index_label=df.index.name,
        )

        conn.commit()

    return inner


def setup_sheets(
    responses_sheet_name: str | None,
    status_sheet_name: str | None,
    sheet_url: str,
    sheets: Sheets,
    restart: bool = False,
):
    def inner(
        exported_file: ExportedFile[pd.DataFrame],
    ):
        survey_id = exported_file.survey_id

        t_responses_sheet_name = format_responses_name(
            survey_id=survey_id, table_name=responses_sheet_name
        )
        t_status_sheet_name = format_status_name(
            survey_id=survey_id, table_name=status_sheet_name
        )

        tmp_sheet_name = str(uuid.uuid4())

        if restart:
            sheets.add(sheet_url, names=tmp_sheet_name)

            sheets.delete(
                sheet_url,
                names=[t_responses_sheet_name, t_status_sheet_name],
                ignore_not_existing=True,
            )

        sheets.add(
            sheet_url,
            names=[t_responses_sheet_name, t_status_sheet_name],
            ignore_existing=True,
        )
        sheets.delete(sheet_url, names=tmp_sheet_name, ignore_not_existing=True)

    return inner


def get_last_status_sheets(
    sheet_name: str | None,
    sheet_url: str,
    sheets: Sheets,
):
    def inner(
        survey_id: str,
    ):
        status_sheet_name = format_status_name(
            survey_id=survey_id, table_name=sheet_name
        )

        status_sheet = SheetsValueRange(
            sheets=sheets, spreadsheet_id=sheet_url, sheet_name=status_sheet_name
        )

        last_status = status_sheet[..., ...].to_frame()
        if last_status is None or last_status.empty:
            return None

        return last_status.iloc[-1].to_dict()

    return inner


def write_status_sheets(
    sheet_name: str | None,
    sheet_url: str,
    sheets: Sheets,
):
    def inner(exported_file: ExportedFile[T]):
        survey_id = exported_file.survey_id
        status_sheet_name = format_status_name(
            survey_id=survey_id, table_name=sheet_name
        )

        row = format_status_row(exported_file)
        return sheets.append(
            spreadsheet_id=sheet_url,
            range_name=status_sheet_name,
            values=[row],
        )

    return inner


def write_responses_sheets(
    sheet_name: str | None,
    sheet_url: str,
    sheets: Sheets,
):
    def inner(exported_file: ExportedFile[pd.DataFrame]):
        df = exported_file.data
        survey_id = exported_file.survey_id

        responses_sheet_name = format_responses_name(
            survey_id=survey_id, table_name=sheet_name
        )

        values = sheets.from_frame(df.reset_index(drop=False), as_dict=True)

        if len(values) == 0:
            return

        sheets.append(
            spreadsheet_id=sheet_url,
            range_name=responses_sheet_name,
            values=values,
        )

    return inner


ResponsePostProcessingFunc = Callable[[pd.DataFrame], pd.DataFrame]

responses_post_processing_func_default: ResponsePostProcessingFunc = lambda x: x


def _sync(
    survey_id: str,
    surveys: Surveys,
    status_reader: Callable[[str], dict | None],
    status_writer: Callable[[ExportedFile[pd.DataFrame]], None],
    responses_writer: Callable[[ExportedFile[pd.DataFrame]], None],
    setup_func: Callable[[ExportedFile[pd.DataFrame]], None],
    responses_post_processing_func: ResponsePostProcessingFunc = responses_post_processing_func_default,
    **kwargs: Any,
):
    survey_id = parse_file_id(survey_id)
    logger.info(f"Syncing survey {survey_id}...")

    last_status = None
    try:
        last_status = status_reader(
            survey_id,
        )
    except Exception as e:
        logger.warning(f"Failed to read last status: {e}")

    last_response_id = (
        last_status["last_response_id"] if last_status is not None else None
    )
    # Only fallback to continuation token if last response ID is None.
    continuation_token = (
        last_status["continuation_token"]
        if last_status is not None and last_response_id is None
        else None
    )

    logger.info(
        f"Last response ID: {last_response_id} and continuation token: {continuation_token}"
    )

    exported_file = surveys.get_responses_df(
        survey_id=survey_id,
        last_response_id=last_response_id,
        continuation_token=continuation_token,
        **kwargs,
    )
    exported_file.data = responses_post_processing_func(exported_file.data)

    logger.info(f"Setting up tables...")
    setup_func(exported_file)

    logger.info(f"Writing responses...")
    responses_writer(exported_file)

    logger.info(f"Writing status...")
    status_writer(exported_file)


def sync_sql(
    survey_id: str,
    surveys: Surveys,
    conn: sqlalchemy.Connection,
    responses_table_name: str | None = None,
    status_table_name: str | None = None,
    restart: bool = False,
    response_post_processing_func: "ResponsePostProcessingFunc" = responses_post_processing_func_default,
    **kwargs: Any,
) -> None:
    survey_id = parse_file_id(survey_id)

    """
    Syncs survey responses and status from a given survey source to a SQL database.

    This requires two tables to be present in the target database:
        - A table to store the survey responses.
        - A table to store the last status of the survey.

    These will be automatically created if they do not exist:
        - If a table name is provided, the responses table will be named as such, and the status table will be named as {table_name}_status.
        - If a table name is not provided, the responses table will be named as {survey_id}_responses and the status table will be named as {survey_id}_status.

    The process is as thus:
        1. The last status of the survey is retrieved from the target.
            1a. If the survey has never been synced, the last status is None.
            1b. If the last status is found, then the last response ID and continuation token are retrieved.
        2. The survey responses are retrieved from the survey source, starting from the last response ID if possible, and the continuation token if possible.
        3. The retrieved survey responses are post-processed using the response_post_processing_func function.
        4. The post-processed survey responses are written to the target.
        5. The last status is written to the target.
    """
    _sync(
        survey_id=survey_id,
        surveys=surveys,
        status_reader=get_last_status_sql(table_name=status_table_name, conn=conn),
        status_writer=write_status_sql(table_name=status_table_name, conn=conn),
        responses_writer=write_responses_sql(
            table_name=responses_table_name, conn=conn
        ),
        setup_func=setup_sql(
            responses_table_name=responses_table_name,
            status_table_name=status_table_name,
            conn=conn,
            restart=restart,
        ),
        responses_post_processing_func=response_post_processing_func,
        **kwargs,
    )


def sync_sheets(
    survey_id: str,
    surveys: Surveys,
    sheets: Sheets,
    sheet_url: str,
    responses_sheet_name: str | None = None,
    status_sheet_name: str | None = None,
    restart: bool = False,
    response_post_processing_func: "ResponsePostProcessingFunc" = responses_post_processing_func_default,
    **kwargs: Any,
) -> None:
    """Syncs survey responses and status from a given survey source to a Google Sheet.

    This requires two tables to be present in the target database:
        - A table to store the survey responses.
        - A table to store the last status of the survey.

    These will be automatically created if they do not exist:
        - If a table name is provided, the responses table will be named as such, and the status table will be named as {table_name}_status.
        - If a table name is not provided, the responses table will be named as {survey_id}_responses and the status table will be named as {survey_id}_status.

    The process is as thus:
        1. The last status of the survey is retrieved from the target.
            1a. If the survey has never been synced, the last status is None.
            1b. If the last status is found, then the last response ID and continuation token are retrieved.
        2. The survey responses are retrieved from the survey source, starting from the last response ID if possible, and the continuation token if possible.
        3. The retrieved survey responses are post-processed using the response_post_processing_func function.
        4. The post-processed survey responses are written to the target.
        5. The last status is written to the target."""
    _sync(
        survey_id=survey_id,
        surveys=surveys,
        status_reader=get_last_status_sheets(
            sheet_name=status_sheet_name, sheet_url=sheet_url, sheets=sheets
        ),
        status_writer=write_status_sheets(
            sheet_name=status_sheet_name, sheet_url=sheet_url, sheets=sheets
        ),
        responses_writer=write_responses_sheets(
            sheet_name=responses_sheet_name, sheet_url=sheet_url, sheets=sheets
        ),
        setup_func=setup_sheets(
            responses_sheet_name=responses_sheet_name,
            status_sheet_name=status_sheet_name,
            sheet_url=sheet_url,
            sheets=sheets,
            restart=restart,
        ),
        responses_post_processing_func=response_post_processing_func,
        **kwargs,
    )


def sync(
    config: dict[str, Any],
    type: SyncType,
    responses_table_name: str | None,
    status_table_name: str | None,
    verbose: bool,
    restart: bool,
):
    qualtrics_config = config["qualtrics"]

    if "api_token" not in qualtrics_config:
        raise ValueError("API token is not provided in the config file.")

    qualtrics_api_token = qualtrics_config["api_token"]
    codebook_path = (
        pathlib.Path(qualtrics_config["codebook_path"])
        if "codebook_path" in qualtrics_config
        else None
    )

    surveys = Surveys(api_token=qualtrics_api_token)

    if "survey_id" not in qualtrics_config:
        raise ValueError("Survey ID is not provided in the config file.")

    survey_id = qualtrics_config["survey_id"]

    survey_args = (
        qualtrics_config["survey_args"] if "survey_args" in qualtrics_config else {}
    )

    logger.info(f"Type: {type}")
    logger.info(f"Restart: {restart}")
    logger.info(f"Responses table name: {responses_table_name}")
    logger.info(f"Status table name: {status_table_name}")
    logger.info(f"Verbose: {verbose}")

    def post_processing_func(df: pd.DataFrame):
        if codebook_path is None:
            return df

        codebook = generate_codebook(codebook_path)
        tmp = coalesce_multiselect(df, codebook=codebook)
        tmp = rename_columns(tmp, codebook=codebook, verbose=verbose)

        return tmp

    if type == SyncType.SHEETS:
        responses_url = config["google"]["urls"]["responses"]

        creds = get_oauth2_creds(config["google"]["credentials_path"])
        sheets = Sheets(creds=creds)

        sync_sheets(
            survey_id=survey_id,
            surveys=surveys,
            response_post_processing_func=post_processing_func,
            responses_sheet_name=responses_table_name,
            status_sheet_name=status_table_name,
            sheet_url=responses_url,
            sheets=sheets,
            restart=restart,
            **survey_args,
        )
    elif type == SyncType.MYSQL:
        engine = create_mysql_engine(
            **config["mysql"],
        )

        with engine.connect() as conn:
            sync_sql(
                survey_id=survey_id,
                surveys=surveys,
                response_post_processing_func=post_processing_func,
                conn=conn,
                responses_table_name=responses_table_name,
                status_table_name=status_table_name,
                restart=restart,
                **survey_args,
            )


def main():
    parser = ArgumentParser(
        description="""Sync Qualtrics survey responses to a target."""
    )
    parser.add_argument(
        "--config",
        type=pathlib.Path,
        required=False,
        default=pathlib.Path("auth/config.toml"),
    )
    parser.add_argument("--verbose", action="store_true")
    parser.add_argument("--restart", action="store_true")
    parser.add_argument("--responses-table-name", required=False)
    parser.add_argument("--status-table-name", required=False)
    parser.add_argument(
        "--type",
        choices=[t for t in SyncType.__members__.values()],
        required=True,
        type=SyncType,
    )
    args = parser.parse_args()

    config = tomllib.loads(args.config.read_text())

    sync(
        config=config,
        type=args.type,
        responses_table_name=args.responses_table_name,
        status_table_name=args.status_table_name,
        verbose=args.verbose,
        restart=args.restart,
    )


if __name__ == "__main__":
    main()



from enum import Enum


class SchemaPropertyDataType(str, Enum):
    EMBEDDEDDATA = "embeddedData"
    METADATA = "metadata"
    QUESTION = "question"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.schema_property_items_properties import SchemaPropertyItemsProperties
    from ..models.survey_answer import SurveyAnswer


T = TypeVar("T", bound="SchemaPropertyItems")


@_attrs_define
class SchemaPropertyItems:
    """
    Attributes:
        type (Union[Unset, str]):
        one_of (Union[Unset, List['SurveyAnswer']]):
        properties (Union[Unset, SchemaPropertyItemsProperties]):
    """

    type: Union[Unset, str] = UNSET
    one_of: Union[Unset, List["SurveyAnswer"]] = UNSET
    properties: Union[Unset, "SchemaPropertyItemsProperties"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type
        one_of: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.one_of, Unset):
            one_of = []
            for one_of_item_data in self.one_of:
                one_of_item = one_of_item_data.to_dict()

                one_of.append(one_of_item)

        properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.properties, Unset):
            properties = self.properties.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if one_of is not UNSET:
            field_dict["oneOf"] = one_of
        if properties is not UNSET:
            field_dict["properties"] = properties

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.schema_property_items_properties import (
            SchemaPropertyItemsProperties,
        )
        from ..models.survey_answer import SurveyAnswer

        d = src_dict.copy()
        type = d.pop("type", UNSET)

        one_of = []
        _one_of = d.pop("oneOf", UNSET)
        for one_of_item_data in _one_of or []:
            one_of_item = SurveyAnswer.from_dict(one_of_item_data)

            one_of.append(one_of_item)

        _properties = d.pop("properties", UNSET)
        properties: Union[Unset, SchemaPropertyItemsProperties]
        if isinstance(_properties, Unset):
            properties = UNSET
        else:
            properties = SchemaPropertyItemsProperties.from_dict(_properties)

        schema_property_items = cls(
            type=type,
            one_of=one_of,
            properties=properties,
        )

        schema_property_items.additional_properties = d
        return schema_property_items

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.get_filters_list_response_result_elements_item import (
        GetFiltersListResponseResultElementsItem,
    )


T = TypeVar("T", bound="GetFiltersListResponseResult")


@_attrs_define
class GetFiltersListResponseResult:
    """
    Attributes:
        elements (List['GetFiltersListResponseResultElementsItem']):
        next_page (Optional[str]):
    """

    elements: List["GetFiltersListResponseResultElementsItem"]
    next_page: Optional[str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        elements = []
        for elements_item_data in self.elements:
            elements_item = elements_item_data.to_dict()

            elements.append(elements_item)

        next_page = self.next_page

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "elements": elements,
                "nextPage": next_page,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.get_filters_list_response_result_elements_item import (
            GetFiltersListResponseResultElementsItem,
        )

        d = src_dict.copy()
        elements = []
        _elements = d.pop("elements")
        for elements_item_data in _elements:
            elements_item = GetFiltersListResponseResultElementsItem.from_dict(
                elements_item_data
            )

            elements.append(elements_item)

        next_page = d.pop("nextPage")

        get_filters_list_response_result = cls(
            elements=elements,
            next_page=next_page,
        )

        get_filters_list_response_result.additional_properties = d
        return get_filters_list_response_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ImportByURLCreationRequest")


@_attrs_define
class ImportByURLCreationRequest:
    """
    Attributes:
        format_ (str): Allowed values are CSV or TSV. See guide before for instructions on how to format csv or tsv
            file.
        file_url (str):
        charset (Union[Unset, str]): The URL of a CSV or TSV file containing the survey responses to import.
    """

    format_: str
    file_url: str
    charset: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        format_ = self.format_
        file_url = self.file_url
        charset = self.charset

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "format": format_,
                "fileUrl": file_url,
            }
        )
        if charset is not UNSET:
            field_dict["charset"] = charset

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        format_ = d.pop("format")

        file_url = d.pop("fileUrl")

        charset = d.pop("charset", UNSET)

        import_by_url_creation_request = cls(
            format_=format_,
            file_url=file_url,
            charset=charset,
        )

        import_by_url_creation_request.additional_properties = d
        return import_by_url_creation_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.request_status import RequestStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExportStatusResponseResult")


@_attrs_define
class ExportStatusResponseResult:
    """
    Attributes:
        percent_complete (float): Indicates current completion of the export, in a percentage.
        status (RequestStatus): Indicates the status of the asynchronous operation.
        file_id (Union[Unset, str]):  Example: 1dc4c492-fbb6-4713-a7ba-bae9b988a965-def.
        continuation_token (Union[Unset, str]): The next continuation token needed to get all responses recorded since
            this export and not included in this export.
    """

    percent_complete: float
    status: RequestStatus
    file_id: Union[Unset, str] = UNSET
    continuation_token: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        percent_complete = self.percent_complete
        status = self.status.value

        file_id = self.file_id
        continuation_token = self.continuation_token

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "percentComplete": percent_complete,
                "status": status,
            }
        )
        if file_id is not UNSET:
            field_dict["fileId"] = file_id
        if continuation_token is not UNSET:
            field_dict["continuationToken"] = continuation_token

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        percent_complete = d.pop("percentComplete")

        status = RequestStatus(d.pop("status"))

        file_id = d.pop("fileId", UNSET)

        continuation_token = d.pop("continuationToken", UNSET)

        export_status_response_result = cls(
            percent_complete=percent_complete,
            status=status,
            file_id=file_id,
            continuation_token=continuation_token,
        )

        export_status_response_result.additional_properties = d
        return export_status_response_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.default_error_response import DefaultErrorResponse
from ...models.survey_unauthorized_response import SurveyUnauthorizedResponse
from ...models.update_response_request import UpdateResponseRequest
from ...models.update_response_response import UpdateResponseResponse
from ...types import Response


def _get_kwargs(
    response_id: str,
    *,
    json_body: UpdateResponseRequest,
) -> Dict[str, Any]:

    pass

    json_json_body = json_body.to_dict()

    return {
        "method": "put",
        "url": "/responses/{responseId}".format(
            responseId=response_id,
        ),
        "json": json_json_body,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[
    Union[DefaultErrorResponse, SurveyUnauthorizedResponse, UpdateResponseResponse]
]:
    if response.status_code == HTTPStatus.OK:
        response_200 = UpdateResponseResponse.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = DefaultErrorResponse.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = SurveyUnauthorizedResponse.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = DefaultErrorResponse.from_dict(response.json())

        return response_500
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[
    Union[DefaultErrorResponse, SurveyUnauthorizedResponse, UpdateResponseResponse]
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    response_id: str,
    *,
    client: AuthenticatedClient,
    json_body: UpdateResponseRequest,
) -> Response[
    Union[DefaultErrorResponse, SurveyUnauthorizedResponse, UpdateResponseResponse]
]:
    """Update Response

     Update embedded data on a survey response
     Embedded data fields [edited in the Data & Analysis tab](https://www.qualtrics.com/support/survey-
    platform/data-and-analysis-module/data/response-editing/) are stored in a separate database from the
    raw responses. Updates stored in this separate edits database will always override the raw value.
    The public update embedded data API only updates the raw response, therefore any updates made via
    this API will not show up if the field has been edited via Data & Analysis. The update will still
    successfully update the raw response and return a 200; however, the edit made from Data & Analysis
    [will need to be removed](https://www.qualtrics.com/support/survey-platform/data-and-analysis-
    module/data/response-editing/#RestoringOriginalData) before your value made via this API are
    visible.

    <!-- theme: info -->
    > ### Changes Viewable Only for Fields in Survey Flow
    > Updates to embedded data are only viewable for fields set in your survey flow. Updates to fields
    not in the flow will still be added to your survey responses, but will be hidden until you update
    your survey.

    Args:
        response_id (str):
        json_body (UpdateResponseRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[DefaultErrorResponse, SurveyUnauthorizedResponse, UpdateResponseResponse]]
    """

    kwargs = _get_kwargs(
        response_id=response_id,
        json_body=json_body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    response_id: str,
    *,
    client: AuthenticatedClient,
    json_body: UpdateResponseRequest,
) -> Optional[
    Union[DefaultErrorResponse, SurveyUnauthorizedResponse, UpdateResponseResponse]
]:
    """Update Response

     Update embedded data on a survey response
     Embedded data fields [edited in the Data & Analysis tab](https://www.qualtrics.com/support/survey-
    platform/data-and-analysis-module/data/response-editing/) are stored in a separate database from the
    raw responses. Updates stored in this separate edits database will always override the raw value.
    The public update embedded data API only updates the raw response, therefore any updates made via
    this API will not show up if the field has been edited via Data & Analysis. The update will still
    successfully update the raw response and return a 200; however, the edit made from Data & Analysis
    [will need to be removed](https://www.qualtrics.com/support/survey-platform/data-and-analysis-
    module/data/response-editing/#RestoringOriginalData) before your value made via this API are
    visible.

    <!-- theme: info -->
    > ### Changes Viewable Only for Fields in Survey Flow
    > Updates to embedded data are only viewable for fields set in your survey flow. Updates to fields
    not in the flow will still be added to your survey responses, but will be hidden until you update
    your survey.

    Args:
        response_id (str):
        json_body (UpdateResponseRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[DefaultErrorResponse, SurveyUnauthorizedResponse, UpdateResponseResponse]
    """

    return sync_detailed(
        response_id=response_id,
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    response_id: str,
    *,
    client: AuthenticatedClient,
    json_body: UpdateResponseRequest,
) -> Response[
    Union[DefaultErrorResponse, SurveyUnauthorizedResponse, UpdateResponseResponse]
]:
    """Update Response

     Update embedded data on a survey response
     Embedded data fields [edited in the Data & Analysis tab](https://www.qualtrics.com/support/survey-
    platform/data-and-analysis-module/data/response-editing/) are stored in a separate database from the
    raw responses. Updates stored in this separate edits database will always override the raw value.
    The public update embedded data API only updates the raw response, therefore any updates made via
    this API will not show up if the field has been edited via Data & Analysis. The update will still
    successfully update the raw response and return a 200; however, the edit made from Data & Analysis
    [will need to be removed](https://www.qualtrics.com/support/survey-platform/data-and-analysis-
    module/data/response-editing/#RestoringOriginalData) before your value made via this API are
    visible.

    <!-- theme: info -->
    > ### Changes Viewable Only for Fields in Survey Flow
    > Updates to embedded data are only viewable for fields set in your survey flow. Updates to fields
    not in the flow will still be added to your survey responses, but will be hidden until you update
    your survey.

    Args:
        response_id (str):
        json_body (UpdateResponseRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[DefaultErrorResponse, SurveyUnauthorizedResponse, UpdateResponseResponse]]
    """

    kwargs = _get_kwargs(
        response_id=response_id,
        json_body=json_body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    response_id: str,
    *,
    client: AuthenticatedClient,
    json_body: UpdateResponseRequest,
) -> Optional[
    Union[DefaultErrorResponse, SurveyUnauthorizedResponse, UpdateResponseResponse]
]:
    """Update Response

     Update embedded data on a survey response
     Embedded data fields [edited in the Data & Analysis tab](https://www.qualtrics.com/support/survey-
    platform/data-and-analysis-module/data/response-editing/) are stored in a separate database from the
    raw responses. Updates stored in this separate edits database will always override the raw value.
    The public update embedded data API only updates the raw response, therefore any updates made via
    this API will not show up if the field has been edited via Data & Analysis. The update will still
    successfully update the raw response and return a 200; however, the edit made from Data & Analysis
    [will need to be removed](https://www.qualtrics.com/support/survey-platform/data-and-analysis-
    module/data/response-editing/#RestoringOriginalData) before your value made via this API are
    visible.

    <!-- theme: info -->
    > ### Changes Viewable Only for Fields in Survey Flow
    > Updates to embedded data are only viewable for fields set in your survey flow. Updates to fields
    not in the flow will still be added to your survey responses, but will be hidden until you update
    your survey.

    Args:
        response_id (str):
        json_body (UpdateResponseRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[DefaultErrorResponse, SurveyUnauthorizedResponse, UpdateResponseResponse]
    """

    return (
        await asyncio_detailed(
            response_id=response_id,
            client=client,
            json_body=json_body,
        )
    ).parsed



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.get_filters_list_response import GetFiltersListResponse
from ...models.survey_not_found import SurveyNotFound
from ...types import Response


def _get_kwargs(
    survey_id: str,
) -> Dict[str, Any]:

    pass

    return {
        "method": "get",
        "url": "/surveys/{surveyId}/filters".format(
            surveyId=survey_id,
        ),
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[GetFiltersListResponse, SurveyNotFound]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = GetFiltersListResponse.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = SurveyNotFound.from_dict(response.json())

        return response_404
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[GetFiltersListResponse, SurveyNotFound]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    survey_id: str,
    *,
    client: AuthenticatedClient,
) -> Response[Union[GetFiltersListResponse, SurveyNotFound]]:
    """Get List of Available Filters

     Gets a list of the available filters for the given survey. Filters are survey specific, and can only
    be set up via the Qualtrics web page for your brand. You can find more information on how to add
    filters at the [Filtering Responses](https://www.qualtrics.com/support/survey-platform/data-and-
    analysis-module/data/filtering-responses/) support page.

    The most important field in the response you get from this call are the returned filterIds. You can
    use these to narrow your exports to responses that fit conditions of your choice.

    Args:
        survey_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[GetFiltersListResponse, SurveyNotFound]]
    """

    kwargs = _get_kwargs(
        survey_id=survey_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    survey_id: str,
    *,
    client: AuthenticatedClient,
) -> Optional[Union[GetFiltersListResponse, SurveyNotFound]]:
    """Get List of Available Filters

     Gets a list of the available filters for the given survey. Filters are survey specific, and can only
    be set up via the Qualtrics web page for your brand. You can find more information on how to add
    filters at the [Filtering Responses](https://www.qualtrics.com/support/survey-platform/data-and-
    analysis-module/data/filtering-responses/) support page.

    The most important field in the response you get from this call are the returned filterIds. You can
    use these to narrow your exports to responses that fit conditions of your choice.

    Args:
        survey_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[GetFiltersListResponse, SurveyNotFound]
    """

    return sync_detailed(
        survey_id=survey_id,
        client=client,
    ).parsed


async def asyncio_detailed(
    survey_id: str,
    *,
    client: AuthenticatedClient,
) -> Response[Union[GetFiltersListResponse, SurveyNotFound]]:
    """Get List of Available Filters

     Gets a list of the available filters for the given survey. Filters are survey specific, and can only
    be set up via the Qualtrics web page for your brand. You can find more information on how to add
    filters at the [Filtering Responses](https://www.qualtrics.com/support/survey-platform/data-and-
    analysis-module/data/filtering-responses/) support page.

    The most important field in the response you get from this call are the returned filterIds. You can
    use these to narrow your exports to responses that fit conditions of your choice.

    Args:
        survey_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[GetFiltersListResponse, SurveyNotFound]]
    """

    kwargs = _get_kwargs(
        survey_id=survey_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    survey_id: str,
    *,
    client: AuthenticatedClient,
) -> Optional[Union[GetFiltersListResponse, SurveyNotFound]]:
    """Get List of Available Filters

     Gets a list of the available filters for the given survey. Filters are survey specific, and can only
    be set up via the Qualtrics web page for your brand. You can find more information on how to add
    filters at the [Filtering Responses](https://www.qualtrics.com/support/survey-platform/data-and-
    analysis-module/data/filtering-responses/) support page.

    The most important field in the response you get from this call are the returned filterIds. You can
    use these to narrow your exports to responses that fit conditions of your choice.

    Args:
        survey_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[GetFiltersListResponse, SurveyNotFound]
    """

    return (
        await asyncio_detailed(
            survey_id=survey_id,
            client=client,
        )
    ).parsed



import argparse
import json
import pathlib
import re
from typing import *

from qualtrics_utils.utils import normalize_html_string


def map_questions(survey_element: dict[str, Any]) -> Optional[dict[str, Any]]:
    PrimaryAttribute: str = survey_element["PrimaryAttribute"]
    Payload: dict[str, Any] = survey_element["Payload"]

    QuestionText: str = Payload["QuestionText"]
    DataExportTag: str = Payload["DataExportTag"]
    QuestionType: str = Payload["QuestionType"]
    QuestionDescription: str = Payload["QuestionDescription"]

    Choices: Optional[dict[str, Any]] = Payload.get("Choices")
    ChoiceOrder: Optional[list[str]] = Payload.get("ChoiceOrder")

    if Choices is not None and ChoiceOrder is not None:
        ChoiceOrder = [str(i) for i in ChoiceOrder]
        Choices = {key: Choices[key] for key in ChoiceOrder}

    Selector: Optional[str] = Payload.get("Selector")
    RecodeValues: dict[str, str] = Payload.get("RecodeValues", {})

    Answers: Optional[dict[str, Any]] = Payload.get("Answers")
    ChoiceDataExportTags: Optional[dict[str, str] | bool] = Payload.get(
        "ChoiceDataExportTags"
    )

    root_q_num: str = DataExportTag
    root_q_str: str = QuestionText

    def merge_a_choices(
        a_choices: dict[str, str], questions: dict[str, dict[str, Any]]
    ) -> list[dict[str, Any]]:
        """Merge answer choices into questions."""
        return [
            {
                **value,
                "answer_choices": a_choices,
            }
            for _, value in questions.items()
        ]

    def create_question(
        q_num: str,
        q_str: str,
        a_choices: Optional[dict[str, str]] = None,
    ) -> dict[str, Any]:
        """A question object is a question number, question string, and an optional series of
        answer choices. We collate and normalize all three here.

        Qualtrics preserves the raw HTML in both question string and answer choice string,
        so we strip that away here.
        """
        out = {
            "question_number": q_num,
            "question_string": q_str,
        }
        if a_choices is not None:
            return {**out, "answer_choices": a_choices}
        else:
            return out

    def explode_question_number() -> dict[str, dict[str, Any]]:
        """If we have choices to pick from, we map those choices to question numbers.
        If a question number is found in the ChoiceDataExportTags object, that takes
        precedence, so we map it to that number."""
        questions = {}
        if Choices is None or ChoiceOrder is None:
            questions["1"] = create_question(root_q_num, root_q_str)
            return questions

        # sort Choices by ChoiceOrder:

        for key, value in Choices.items():
            sub_q_num = RecodeValues.get(key, key)
            q_num = f"{root_q_num}_{sub_q_num}"

            value = Choices[key]
            q_str = value["Display"]

            if isinstance(ChoiceDataExportTags, dict):
                q_num = ChoiceDataExportTags.get(key, q_num)

            if QuestionType != "TE" and value.get("TextEntry") is not None:
                questions[key] = create_question(q_num, q_str)

                q_num += "_TEXT"
                key += "_TEXT"

                questions[key] = create_question(q_num, q_str)
            else:
                questions[key] = create_question(q_num, q_str)

        return questions

    def process_question() -> Optional[list[dict[str, Any]]]:
        if QuestionType == "Matrix":
            """A 2-d matrix option has one column of questions, then a series of columns as answers.
            For each question in the question column, we merge the answer choices, creating the final 2-d
            result.

            Example in 2-d form:
                a0  a1  a2
            q0  00  00  00
            q1  00  00  00

            Then as a pseudo-dict:

            q0: {a0: 00, a1: 00, a2: 00}
            q1: {a0: 00, a1: 00, a2: 00}
            """
            a_choices = {
                RecodeValues.get(key, key): value["Display"]
                for key, value in Answers.items()  # type: ignore
            }
            questions = explode_question_number()
            return merge_a_choices(a_choices, questions)
        # Multiple choice
        elif QuestionType == "MC":
            # Single answer
            if Selector == "DL" or Selector == "SAVR":
                a_choices = {
                    RecodeValues.get(key, key): value["Display"]
                    for key, value in Choices.items()  # type: ignore
                }
                return [create_question(root_q_num, root_q_str, a_choices)]
            # Multiple answer, which creates a sort-of 2-d matrix.
            elif Selector == "MAVR":
                a_choices = {"-1": "Not Selected", "1": "Selected", "Null": "Not Shown"}
                questions = explode_question_number()
                return merge_a_choices(a_choices, questions)
        # Text entry, so no answer choices here.
        elif QuestionType == "TE":
            questions = explode_question_number()
            return list(questions.values())

        return None

    return {
        "question_number": root_q_num,
        "question_string": root_q_str,
        "question_type": QuestionType,
        "questions": process_question(),
    }


def format_codebook(codebook: list[dict[str, Any]]) -> list[dict[str, Any]]:
    """Formats the codebook to be more readable.

    Sorts by question number, and then normalizes the HTML
    in the question string and answer choices."""

    # sort by question number
    codebook.sort(key=lambda x: float(str(x["question_number"]).lstrip("Q")))

    for root_question in codebook:
        root_q_str = root_question["question_string"]
        root_question["question_string"] = normalize_html_string(root_q_str)

        if root_question["questions"] is None:
            continue

        for question in root_question["questions"]:
            q_str, a_choices = (
                question.get("question_string", ""),
                question.get("answer_choices"),
            )
            question["question_string"] = normalize_html_string(q_str)

            if a_choices is not None:
                a_choices = {
                    key: normalize_html_string(value)
                    for key, value in a_choices.items()
                }
                question["answer_choices"] = a_choices

    return codebook


def generate_codebook(filepath: pathlib.Path) -> list[dict[str, Any]]:
    """Generates a codebook from a Qualtrics .qsf file.

    For more information on .qsf files, see:
    https://api.qualtrics.com/docs/qsf

    Args:
        filepath (pathlib.Path): The path to the .qsf file.
    """
    codebook: list[dict[str, Any]] = []

    with open(filepath, "r") as file:
        qsf_json = json.load(file)

        SurveyElements = qsf_json["SurveyElements"]

        for element in SurveyElements:
            Element = element["Element"]

            if Element == "SQ":
                questions = map_questions(element)
                if questions is not None:
                    codebook.append(questions)

    codebook = format_codebook(codebook)

    return codebook


def main() -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument("input", help="Input file path.", type=pathlib.Path)
    args = parser.parse_args()

    filepath = args.input

    if filepath.suffix != ".qsf":
        raise ValueError("Please input a valid .qsf file.")

    out_path = filepath.parent / f"{filepath.stem}-codebook.json"

    codebook = generate_codebook(filepath)

    with open(out_path, "w") as file:
        json.dump(codebook, file, indent=4)


if __name__ == "__main__":
    main()



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.schema_property import SchemaProperty


T = TypeVar("T", bound="GetSchemaResponseResultPropertiesValuesProperties")


@_attrs_define
class GetSchemaResponseResultPropertiesValuesProperties:
    """
    Attributes:
        distribution_channel (Union[Unset, SchemaProperty]): Each entry represents a field in the `values` field of the
            JSON survey response
        duration (Union[Unset, SchemaProperty]): Each entry represents a field in the `values` field of the JSON survey
            response
        end_date (Union[Unset, SchemaProperty]): Each entry represents a field in the `values` field of the JSON survey
            response
        finished (Union[Unset, SchemaProperty]): Each entry represents a field in the `values` field of the JSON survey
            response
        location_latitude (Union[Unset, SchemaProperty]): Each entry represents a field in the `values` field of the
            JSON survey response
        location_longitude (Union[Unset, SchemaProperty]): Each entry represents a field in the `values` field of the
            JSON survey response
        progress (Union[Unset, SchemaProperty]): Each entry represents a field in the `values` field of the JSON survey
            response
        recorded_date (Union[Unset, SchemaProperty]): Each entry represents a field in the `values` field of the JSON
            survey response
        start_date (Union[Unset, SchemaProperty]): Each entry represents a field in the `values` field of the JSON
            survey response
        status (Union[Unset, SchemaProperty]): Each entry represents a field in the `values` field of the JSON survey
            response
        user_language (Union[Unset, SchemaProperty]): Each entry represents a field in the `values` field of the JSON
            survey response
    """

    distribution_channel: Union[Unset, "SchemaProperty"] = UNSET
    duration: Union[Unset, "SchemaProperty"] = UNSET
    end_date: Union[Unset, "SchemaProperty"] = UNSET
    finished: Union[Unset, "SchemaProperty"] = UNSET
    location_latitude: Union[Unset, "SchemaProperty"] = UNSET
    location_longitude: Union[Unset, "SchemaProperty"] = UNSET
    progress: Union[Unset, "SchemaProperty"] = UNSET
    recorded_date: Union[Unset, "SchemaProperty"] = UNSET
    start_date: Union[Unset, "SchemaProperty"] = UNSET
    status: Union[Unset, "SchemaProperty"] = UNSET
    user_language: Union[Unset, "SchemaProperty"] = UNSET
    additional_properties: Dict[str, "SchemaProperty"] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        distribution_channel: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.distribution_channel, Unset):
            distribution_channel = self.distribution_channel.to_dict()

        duration: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.duration, Unset):
            duration = self.duration.to_dict()

        end_date: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.to_dict()

        finished: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.finished, Unset):
            finished = self.finished.to_dict()

        location_latitude: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.location_latitude, Unset):
            location_latitude = self.location_latitude.to_dict()

        location_longitude: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.location_longitude, Unset):
            location_longitude = self.location_longitude.to_dict()

        progress: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.progress, Unset):
            progress = self.progress.to_dict()

        recorded_date: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.recorded_date, Unset):
            recorded_date = self.recorded_date.to_dict()

        start_date: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.to_dict()

        status: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.to_dict()

        user_language: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.user_language, Unset):
            user_language = self.user_language.to_dict()

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop.to_dict()

        field_dict.update({})
        if distribution_channel is not UNSET:
            field_dict["distributionChannel"] = distribution_channel
        if duration is not UNSET:
            field_dict["duration"] = duration
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if finished is not UNSET:
            field_dict["finished"] = finished
        if location_latitude is not UNSET:
            field_dict["locationLatitude"] = location_latitude
        if location_longitude is not UNSET:
            field_dict["locationLongitude"] = location_longitude
        if progress is not UNSET:
            field_dict["progress"] = progress
        if recorded_date is not UNSET:
            field_dict["recordedDate"] = recorded_date
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if status is not UNSET:
            field_dict["status"] = status
        if user_language is not UNSET:
            field_dict["userLanguage"] = user_language

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.schema_property import SchemaProperty

        d = src_dict.copy()
        _distribution_channel = d.pop("distributionChannel", UNSET)
        distribution_channel: Union[Unset, SchemaProperty]
        if isinstance(_distribution_channel, Unset):
            distribution_channel = UNSET
        else:
            distribution_channel = SchemaProperty.from_dict(_distribution_channel)

        _duration = d.pop("duration", UNSET)
        duration: Union[Unset, SchemaProperty]
        if isinstance(_duration, Unset):
            duration = UNSET
        else:
            duration = SchemaProperty.from_dict(_duration)

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, SchemaProperty]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = SchemaProperty.from_dict(_end_date)

        _finished = d.pop("finished", UNSET)
        finished: Union[Unset, SchemaProperty]
        if isinstance(_finished, Unset):
            finished = UNSET
        else:
            finished = SchemaProperty.from_dict(_finished)

        _location_latitude = d.pop("locationLatitude", UNSET)
        location_latitude: Union[Unset, SchemaProperty]
        if isinstance(_location_latitude, Unset):
            location_latitude = UNSET
        else:
            location_latitude = SchemaProperty.from_dict(_location_latitude)

        _location_longitude = d.pop("locationLongitude", UNSET)
        location_longitude: Union[Unset, SchemaProperty]
        if isinstance(_location_longitude, Unset):
            location_longitude = UNSET
        else:
            location_longitude = SchemaProperty.from_dict(_location_longitude)

        _progress = d.pop("progress", UNSET)
        progress: Union[Unset, SchemaProperty]
        if isinstance(_progress, Unset):
            progress = UNSET
        else:
            progress = SchemaProperty.from_dict(_progress)

        _recorded_date = d.pop("recordedDate", UNSET)
        recorded_date: Union[Unset, SchemaProperty]
        if isinstance(_recorded_date, Unset):
            recorded_date = UNSET
        else:
            recorded_date = SchemaProperty.from_dict(_recorded_date)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, SchemaProperty]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = SchemaProperty.from_dict(_start_date)

        _status = d.pop("status", UNSET)
        status: Union[Unset, SchemaProperty]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = SchemaProperty.from_dict(_status)

        _user_language = d.pop("userLanguage", UNSET)
        user_language: Union[Unset, SchemaProperty]
        if isinstance(_user_language, Unset):
            user_language = UNSET
        else:
            user_language = SchemaProperty.from_dict(_user_language)

        get_schema_response_result_properties_values_properties = cls(
            distribution_channel=distribution_channel,
            duration=duration,
            end_date=end_date,
            finished=finished,
            location_latitude=location_latitude,
            location_longitude=location_longitude,
            progress=progress,
            recorded_date=recorded_date,
            start_date=start_date,
            status=status,
            user_language=user_language,
        )

        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = SchemaProperty.from_dict(prop_dict)

            additional_properties[prop_name] = additional_property

        get_schema_response_result_properties_values_properties.additional_properties = (
            additional_properties
        )
        return get_schema_response_result_properties_values_properties

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> "SchemaProperty":
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: "SchemaProperty") -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from dataclasses import dataclass
from typing import Callable, Generic, TypeVar
from datetime import datetime

HEADERS = {"X-API-TOKEN": "", "Content-Type": "application/json"}

VERSION = "v3"

BASE_URL: Callable[
    [str], str
] = lambda version: f"https://yul1.qualtrics.com/API/{version}/"

T = TypeVar("T")


@dataclass
class ExportedFile(Generic[T]):
    survey_id: str
    file_id: str

    continuation_token: str | None
    last_response_id: str | None

    timestamp: datetime

    data: T



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.export_status_response import ExportStatusResponse
from ...models.meta import Meta
from ...types import Response


def _get_kwargs(
    survey_id: str,
    export_progress_id: str,
) -> Dict[str, Any]:

    pass

    return {
        "method": "get",
        "url": "/surveys/{surveyId}/export-responses/{exportProgressId}".format(
            surveyId=survey_id,
            exportProgressId=export_progress_id,
        ),
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[ExportStatusResponse, Meta]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = ExportStatusResponse.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = Meta.from_dict(response.json())

        return response_404
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[ExportStatusResponse, Meta]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    survey_id: str,
    export_progress_id: str,
    *,
    client: AuthenticatedClient,
) -> Response[Union[ExportStatusResponse, Meta]]:
    r"""Get Response Export Progress

     Retrieves the status of a response export for the given `exportProgressId`. Please note you will
    find the `exportProgressId` in the Export Responses POST response in the `progressId` field.

    If you receive a status of `failed`, record your `requestId` and try your export again. Contact
    support with your `requestId` and date and time of the call to troubleshoot.

    <!-- theme: danger -->
    > ### Before calling this API, make sure your `exportProgressId` is not null.
    > You could end up in an infinite loop if you don't code this properly. Please check for null before
    making this call. Also check for a 404 response and stop polling if you receive one.

    <!-- theme: info -->
    >### `percentComplete` Field
    >Please note the percentComplete field is a convenience field. You cannot download the file until
    the \"status\" field indicates `complete` - which is also when you'll receive a fileid.


    Args:
        survey_id (str):
        export_progress_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ExportStatusResponse, Meta]]
    """

    kwargs = _get_kwargs(
        survey_id=survey_id,
        export_progress_id=export_progress_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    survey_id: str,
    export_progress_id: str,
    *,
    client: AuthenticatedClient,
) -> Optional[Union[ExportStatusResponse, Meta]]:
    r"""Get Response Export Progress

     Retrieves the status of a response export for the given `exportProgressId`. Please note you will
    find the `exportProgressId` in the Export Responses POST response in the `progressId` field.

    If you receive a status of `failed`, record your `requestId` and try your export again. Contact
    support with your `requestId` and date and time of the call to troubleshoot.

    <!-- theme: danger -->
    > ### Before calling this API, make sure your `exportProgressId` is not null.
    > You could end up in an infinite loop if you don't code this properly. Please check for null before
    making this call. Also check for a 404 response and stop polling if you receive one.

    <!-- theme: info -->
    >### `percentComplete` Field
    >Please note the percentComplete field is a convenience field. You cannot download the file until
    the \"status\" field indicates `complete` - which is also when you'll receive a fileid.


    Args:
        survey_id (str):
        export_progress_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ExportStatusResponse, Meta]
    """

    return sync_detailed(
        survey_id=survey_id,
        export_progress_id=export_progress_id,
        client=client,
    ).parsed


async def asyncio_detailed(
    survey_id: str,
    export_progress_id: str,
    *,
    client: AuthenticatedClient,
) -> Response[Union[ExportStatusResponse, Meta]]:
    r"""Get Response Export Progress

     Retrieves the status of a response export for the given `exportProgressId`. Please note you will
    find the `exportProgressId` in the Export Responses POST response in the `progressId` field.

    If you receive a status of `failed`, record your `requestId` and try your export again. Contact
    support with your `requestId` and date and time of the call to troubleshoot.

    <!-- theme: danger -->
    > ### Before calling this API, make sure your `exportProgressId` is not null.
    > You could end up in an infinite loop if you don't code this properly. Please check for null before
    making this call. Also check for a 404 response and stop polling if you receive one.

    <!-- theme: info -->
    >### `percentComplete` Field
    >Please note the percentComplete field is a convenience field. You cannot download the file until
    the \"status\" field indicates `complete` - which is also when you'll receive a fileid.


    Args:
        survey_id (str):
        export_progress_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ExportStatusResponse, Meta]]
    """

    kwargs = _get_kwargs(
        survey_id=survey_id,
        export_progress_id=export_progress_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    survey_id: str,
    export_progress_id: str,
    *,
    client: AuthenticatedClient,
) -> Optional[Union[ExportStatusResponse, Meta]]:
    r"""Get Response Export Progress

     Retrieves the status of a response export for the given `exportProgressId`. Please note you will
    find the `exportProgressId` in the Export Responses POST response in the `progressId` field.

    If you receive a status of `failed`, record your `requestId` and try your export again. Contact
    support with your `requestId` and date and time of the call to troubleshoot.

    <!-- theme: danger -->
    > ### Before calling this API, make sure your `exportProgressId` is not null.
    > You could end up in an infinite loop if you don't code this properly. Please check for null before
    making this call. Also check for a 404 response and stop polling if you receive one.

    <!-- theme: info -->
    >### `percentComplete` Field
    >Please note the percentComplete field is a convenience field. You cannot download the file until
    the \"status\" field indicates `complete` - which is also when you'll receive a fileid.


    Args:
        survey_id (str):
        export_progress_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ExportStatusResponse, Meta]
    """

    return (
        await asyncio_detailed(
            survey_id=survey_id,
            export_progress_id=export_progress_id,
            client=client,
        )
    ).parsed



from enum import Enum


class GetSchemaResponseResultPropertiesDisplayedValuesAdditionalPropertiesType(
    str, Enum
):
    ARRAY = "array"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class GetSchemaResponseResultSchema(str, Enum):
    HTTPSJSON_SCHEMA_ORGDRAFT2019_09SCHEMA = (
        "https://json-schema.org/draft/2019-09/schema"
    )

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.request_status import RequestStatus

T = TypeVar("T", bound="ImportStatusResponseResult")


@_attrs_define
class ImportStatusResponseResult:
    """
    Attributes:
        percent_complete (float):
        status (RequestStatus): Indicates the status of the asynchronous operation.
    """

    percent_complete: float
    status: RequestStatus
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        percent_complete = self.percent_complete
        status = self.status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "percentComplete": percent_complete,
                "status": status,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        percent_complete = d.pop("percentComplete")

        status = RequestStatus(d.pop("status"))

        import_status_response_result = cls(
            percent_complete=percent_complete,
            status=status,
        )

        import_status_response_result.additional_properties = d
        return import_status_response_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.create_response_request_values import CreateResponseRequestValues


T = TypeVar("T", bound="CreateResponseRequest")


@_attrs_define
class CreateResponseRequest:
    """
    Attributes:
        values (CreateResponseRequestValues): `values` is a set of key-value pairs corresponding to question-answer
            pairs. There can also be key-value pairs representing meta data for the response such as `startDate` or
            `userLanguage`.
    """

    values: "CreateResponseRequestValues"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        values = self.values.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "values": values,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.create_response_request_values import CreateResponseRequestValues

        d = src_dict.copy()
        values = CreateResponseRequestValues.from_dict(d.pop("values"))

        create_response_request = cls(
            values=values,
        )

        create_response_request.additional_properties = d
        return create_response_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.meta_error import MetaError


T = TypeVar("T", bound="Meta")


@_attrs_define
class Meta:
    """
    Attributes:
        request_id (str):
        http_status (str):
        notice (Union[Unset, str]):
        error (Union[Unset, MetaError]):
    """

    request_id: str
    http_status: str
    notice: Union[Unset, str] = UNSET
    error: Union[Unset, "MetaError"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        request_id = self.request_id
        http_status = self.http_status
        notice = self.notice
        error: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.error, Unset):
            error = self.error.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "requestId": request_id,
                "httpStatus": http_status,
            }
        )
        if notice is not UNSET:
            field_dict["notice"] = notice
        if error is not UNSET:
            field_dict["error"] = error

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.meta_error import MetaError

        d = src_dict.copy()
        request_id = d.pop("requestId")

        http_status = d.pop("httpStatus")

        notice = d.pop("notice", UNSET)

        _error = d.pop("error", UNSET)
        error: Union[Unset, MetaError]
        if isinstance(_error, Unset):
            error = UNSET
        else:
            error = MetaError.from_dict(_error)

        meta = cls(
            request_id=request_id,
            http_status=http_status,
            notice=notice,
            error=error,
        )

        meta.additional_properties = d
        return meta

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.creation_response import CreationResponse
from ...models.export_creation_request import ExportCreationRequest
from ...models.meta import Meta
from ...models.survey_not_found import SurveyNotFound
from ...types import Response


def _get_kwargs(
    survey_id: str,
    *,
    json_body: ExportCreationRequest,
) -> Dict[str, Any]:

    pass

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": "/surveys/{surveyId}/export-responses".format(
            surveyId=survey_id,
        ),
        "json": json_json_body,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[CreationResponse, Meta, SurveyNotFound]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = CreationResponse.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = Meta.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = SurveyNotFound.from_dict(response.json())

        return response_404
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[CreationResponse, Meta, SurveyNotFound]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    survey_id: str,
    *,
    client: AuthenticatedClient,
    json_body: ExportCreationRequest,
) -> Response[Union[CreationResponse, Meta, SurveyNotFound]]:
    """Start Response Export

     Starts an export of a survey's responses. See the [Response Import/Export API
    Overview](../../../../reference/responseImportsExports.json) for more detail on how to use this
    endpoint within a workflow.

    This page will provide additional details and things you should know about this endpoint.

    <!-- theme: danger -->
    > ### If this call doesn't return an HTTP 200 status code, do not start polling for progress.

    <!-- theme: warning -->
    >### Max File Size
    >Currently, response exports exceeding 1.8 GB will fail. To prevent your export from failing, use
    proper limits and filters to limit the size of your final export file.

    <!-- theme: warning -->
    >### Invalid Parameters with JSON and NDJSON
    >When exporting responses as JSON, not all parameters are meaningful. Consequently, the following
    parameters are not allowed when starting a JSON or NDJSON export:
    >
    >`includeDisplayOrder`
    >`useLabels`
    >`formatDecimalAsComma`
    >`seenUnansweredRecode`
    >`multiselectSeenUnansweredRecode`
    >`timeZone`
    >`newlineReplacement`
    >`breakoutSets`


    Args:
        survey_id (str):
        json_body (ExportCreationRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[CreationResponse, Meta, SurveyNotFound]]
    """

    kwargs = _get_kwargs(
        survey_id=survey_id,
        json_body=json_body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    survey_id: str,
    *,
    client: AuthenticatedClient,
    json_body: ExportCreationRequest,
) -> Optional[Union[CreationResponse, Meta, SurveyNotFound]]:
    """Start Response Export

     Starts an export of a survey's responses. See the [Response Import/Export API
    Overview](../../../../reference/responseImportsExports.json) for more detail on how to use this
    endpoint within a workflow.

    This page will provide additional details and things you should know about this endpoint.

    <!-- theme: danger -->
    > ### If this call doesn't return an HTTP 200 status code, do not start polling for progress.

    <!-- theme: warning -->
    >### Max File Size
    >Currently, response exports exceeding 1.8 GB will fail. To prevent your export from failing, use
    proper limits and filters to limit the size of your final export file.

    <!-- theme: warning -->
    >### Invalid Parameters with JSON and NDJSON
    >When exporting responses as JSON, not all parameters are meaningful. Consequently, the following
    parameters are not allowed when starting a JSON or NDJSON export:
    >
    >`includeDisplayOrder`
    >`useLabels`
    >`formatDecimalAsComma`
    >`seenUnansweredRecode`
    >`multiselectSeenUnansweredRecode`
    >`timeZone`
    >`newlineReplacement`
    >`breakoutSets`


    Args:
        survey_id (str):
        json_body (ExportCreationRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[CreationResponse, Meta, SurveyNotFound]
    """

    return sync_detailed(
        survey_id=survey_id,
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    survey_id: str,
    *,
    client: AuthenticatedClient,
    json_body: ExportCreationRequest,
) -> Response[Union[CreationResponse, Meta, SurveyNotFound]]:
    """Start Response Export

     Starts an export of a survey's responses. See the [Response Import/Export API
    Overview](../../../../reference/responseImportsExports.json) for more detail on how to use this
    endpoint within a workflow.

    This page will provide additional details and things you should know about this endpoint.

    <!-- theme: danger -->
    > ### If this call doesn't return an HTTP 200 status code, do not start polling for progress.

    <!-- theme: warning -->
    >### Max File Size
    >Currently, response exports exceeding 1.8 GB will fail. To prevent your export from failing, use
    proper limits and filters to limit the size of your final export file.

    <!-- theme: warning -->
    >### Invalid Parameters with JSON and NDJSON
    >When exporting responses as JSON, not all parameters are meaningful. Consequently, the following
    parameters are not allowed when starting a JSON or NDJSON export:
    >
    >`includeDisplayOrder`
    >`useLabels`
    >`formatDecimalAsComma`
    >`seenUnansweredRecode`
    >`multiselectSeenUnansweredRecode`
    >`timeZone`
    >`newlineReplacement`
    >`breakoutSets`


    Args:
        survey_id (str):
        json_body (ExportCreationRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[CreationResponse, Meta, SurveyNotFound]]
    """

    kwargs = _get_kwargs(
        survey_id=survey_id,
        json_body=json_body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    survey_id: str,
    *,
    client: AuthenticatedClient,
    json_body: ExportCreationRequest,
) -> Optional[Union[CreationResponse, Meta, SurveyNotFound]]:
    """Start Response Export

     Starts an export of a survey's responses. See the [Response Import/Export API
    Overview](../../../../reference/responseImportsExports.json) for more detail on how to use this
    endpoint within a workflow.

    This page will provide additional details and things you should know about this endpoint.

    <!-- theme: danger -->
    > ### If this call doesn't return an HTTP 200 status code, do not start polling for progress.

    <!-- theme: warning -->
    >### Max File Size
    >Currently, response exports exceeding 1.8 GB will fail. To prevent your export from failing, use
    proper limits and filters to limit the size of your final export file.

    <!-- theme: warning -->
    >### Invalid Parameters with JSON and NDJSON
    >When exporting responses as JSON, not all parameters are meaningful. Consequently, the following
    parameters are not allowed when starting a JSON or NDJSON export:
    >
    >`includeDisplayOrder`
    >`useLabels`
    >`formatDecimalAsComma`
    >`seenUnansweredRecode`
    >`multiselectSeenUnansweredRecode`
    >`timeZone`
    >`newlineReplacement`
    >`breakoutSets`


    Args:
        survey_id (str):
        json_body (ExportCreationRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[CreationResponse, Meta, SurveyNotFound]
    """

    return (
        await asyncio_detailed(
            survey_id=survey_id,
            client=client,
            json_body=json_body,
        )
    ).parsed



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="RetrieveResponseResponseResultValues")


@_attrs_define
class RetrieveResponseResponseResultValues:
    """`values` is a set of key-value pairs corresponding to question-answer pairs. There can also be key-value pairs
    representing meta data for the response such as `startDate` or `userLanguage`.

        Attributes:
            end_date (datetime.datetime): The point in time when the survey response was finished
            finished (int): If the respondent finished and submitted the survey, the value will be 1, otherwise it will be 0
            recorded_date (datetime.datetime): The point in time when the survey response was recorded
            start_date (datetime.datetime): The point in time when the survey response was started
            status (int): The type of response
            distribution_channel (Union[Unset, str]): The method by which the survey was distributed to respondents
            duration (Union[Unset, int]): How long it took for the respondent to finish the survey in seconds
            location_latitude (Union[Unset, str]): The approximate location of the respondent at the time the survey was
                taken
            location_longitude (Union[Unset, str]): The approximate location of the respondent at the time the survey was
                taken
            progress (Union[Unset, int]): How far the respondent has progressed through the survey as a percentage
            user_language (Union[Unset, str]): The language of the respondent
    """

    end_date: datetime.datetime
    finished: int
    recorded_date: datetime.datetime
    start_date: datetime.datetime
    status: int
    distribution_channel: Union[Unset, str] = UNSET
    duration: Union[Unset, int] = UNSET
    location_latitude: Union[Unset, str] = UNSET
    location_longitude: Union[Unset, str] = UNSET
    progress: Union[Unset, int] = UNSET
    user_language: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        end_date = self.end_date.isoformat()

        finished = self.finished
        recorded_date = self.recorded_date.isoformat()

        start_date = self.start_date.isoformat()

        status = self.status
        distribution_channel = self.distribution_channel
        duration = self.duration
        location_latitude = self.location_latitude
        location_longitude = self.location_longitude
        progress = self.progress
        user_language = self.user_language

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "endDate": end_date,
                "finished": finished,
                "recordedDate": recorded_date,
                "startDate": start_date,
                "status": status,
            }
        )
        if distribution_channel is not UNSET:
            field_dict["distributionChannel"] = distribution_channel
        if duration is not UNSET:
            field_dict["duration"] = duration
        if location_latitude is not UNSET:
            field_dict["locationLatitude"] = location_latitude
        if location_longitude is not UNSET:
            field_dict["locationLongitude"] = location_longitude
        if progress is not UNSET:
            field_dict["progress"] = progress
        if user_language is not UNSET:
            field_dict["userLanguage"] = user_language

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        end_date = isoparse(d.pop("endDate"))

        finished = d.pop("finished")

        recorded_date = isoparse(d.pop("recordedDate"))

        start_date = isoparse(d.pop("startDate"))

        status = d.pop("status")

        distribution_channel = d.pop("distributionChannel", UNSET)

        duration = d.pop("duration", UNSET)

        location_latitude = d.pop("locationLatitude", UNSET)

        location_longitude = d.pop("locationLongitude", UNSET)

        progress = d.pop("progress", UNSET)

        user_language = d.pop("userLanguage", UNSET)

        retrieve_response_response_result_values = cls(
            end_date=end_date,
            finished=finished,
            recorded_date=recorded_date,
            start_date=start_date,
            status=status,
            distribution_channel=distribution_channel,
            duration=duration,
            location_latitude=location_latitude,
            location_longitude=location_longitude,
            progress=progress,
            user_language=user_language,
        )

        retrieve_response_response_result_values.additional_properties = d
        return retrieve_response_response_result_values

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.export_creation_request_format import ExportCreationRequestFormat
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExportCreationRequest")


@_attrs_define
class ExportCreationRequest:
    """
    Attributes:
        format_ (ExportCreationRequestFormat): The format of the export file. This can be one of: `csv`, `tsv`, `json`,
            `ndjson`, `spss`, or `xml` Default: ExportCreationRequestFormat.CSV. Example: csv.
        breakout_sets (Union[Unset, bool]): If true, split multi-value fields into columns. Default: True. Example:
            true.
        compress (Union[Unset, bool]): Compress the final export file as a ZIP file. Exporting without compression is
            only recommended for small exports. Large exports may encounter issues when downloading the large files. It is
            possible in the future that if an export size is too large uncompressed, this flag will be ignored and it will
            be compressed anyway. Default: True. Example: true.
        end_date (Union[Unset, datetime.datetime]): Only export responses recorded before the specified date unless
            `sortByLastModifiedDate` is true. In which case only export responses created or modified before the date. See
            <a href="https://api.qualtrics.com/api-reference/docs/Instructions/dates-and-times.md">Dates and Times</a> for
            more information on the date and time format. The end date is exclusive. Default:
            isoparse('2100-01-01T01:00:00Z'). Example: 2040-02-01T01:50:00Z.
        export_responses_in_progress (Union[Unset, bool]): Only export responses not yet complete. Note that
            `sortByLastModifiedDate` doesn't have any impact when this is set to `true`. Default: True. Example: false.
        filter_id (Union[Unset, str]): If you provide a `filterId`, the export will only return responses matching the
            corresponding filter. For more information on filters and how to create them, see <a
            href="https://www.qualtrics.com/support/survey-platform/data-and-analysis-module/data/filtering-
            responses/">Filtering Responses</a> at Qualtrics Support, or read the <a
            href="https://api.qualtrics.com/instructions/YXBpOjYwOTI5-surveys-response-import-export-api#export-using-a-
            filter">response import/export API overview section on using filters</a>. Example:
            9b67fbb7-ef89-430d-8ddf-f17f44de9254.
        format_decimal_as_comma (Union[Unset, bool]): If `true`, use a comma as a decimal separator instead of a period.
            Default: True. Example: false.
        include_display_order (Union[Unset, bool]): If `true`, include display order information in your export. This is
            useful for surveys with randomization. Note that if used alongside the `embeddedDataIds`, `questionIds`, or
            `surveyMetadataIds` parameters, this will only apply to the display order information for questions. Flow and
            block display order information will always be included. Default: True. Example: false.
        limit (Union[Unset, int]): Maximum number of responses to export. Example: 50.
        multiselect_seen_unanswered_recode (Union[Unset, int]): Recode seen-but-unanswered choices for multi-select
            questions with this value. If not set, this will be the `seenUnansweredRecode` value. Example: 99.
        newline_replacement (Union[Unset, str]): If set, replace newline characters in survey responses with this value.
            Note: this parameter is only relevant for CSV and TSV.
        seen_unanswered_recode (Union[Unset, int]): Recode seen-but-unanswered questions with this value. Example: 0.
        start_date (Union[Unset, datetime.datetime]): Only export responses recorded after the specified date unless
            `sortByLastModifiedDate` is true. In which case only export responses created or modified after the date. See <a
            href="https://api.qualtrics.com/api-reference/docs/Instructions/dates-and-times.md">Dates and Times</a> for more
            information on the date and time format. The start date is inclusive. Default: isoparse('1970-01-01T01:00:00Z').
            Example: 2018-01-01T01:50:00Z.
        time_zone (Union[Unset, str]): Timezone used to determine response date values. Data is recorded in UTC, and
            translates that value to your local time. This can be useful for local survey deadlines. See <a
            href="https://api.qualtrics.com/api-reference/docs/Instructions/dates-and-times.md">Dates and Times</a> for more
            information on time zone format. If this parameter is not provided, dates will be exported in UTC/GMT. Default:
            'UTC'. Example: America/Chicago.
        use_labels (Union[Unset, bool]): Instead of exporting the recode (numeric) value for the answer choice, export
            the text of the answer choice. For more information on recode values, see <a
            href="https://www.qualtrics.com/support/survey-platform/survey-module/question-options/recode-values/">Recode
            Values</a> on the Qualtrics Support Page. Default: True. Example: false.
        embedded_data_ids (Union[Unset, List[str]]): If provided, only export embedded data fields from the provided
            list of Embedded Data IDs. For more information see <a href="https://www.qualtrics.com/support/survey-
            platform/survey-module/survey-flow/standard-elements/embedded-data/">Embedded Data</a>
        question_ids (Union[Unset, List[str]]): If provided, only export answers from the provided list of Question IDs
        survey_metadata_ids (Union[Unset, List[str]]): If provided, only export metadata fields from the provided list
            of Metadata IDs. This will remove metadata included in export by default. The list of those fields is below:
            </dt><dd>
                <dl>
                    <dt><b>startDate</b></dt><dd>Start Date</dd>
                    <dt><b>endDate</b></dt><dd>End Date</dd>
                    <dt><b>status</b></dt><dd>Survey Response type (info found <a
            href="https://www.qualtrics.com/support/survey-platform/data-and-analysis-module/data/download-
            data/understanding-your-dataset/#RespondentInformation">here</a>)</dd>
                    <dt><b>ipAddress</b></dt><dd>IP Address</dd>
                    <dt><b>progress</b></dt><dd>Progress (percentage)</dd>
                    <dt><b>duration</b></dt><dd>Duration (in seconds)</dd>
                    <dt><b>finished</b></dt><dd>boolean for whether or not the survey is complete</dd>
                    <dt><b>recordedDate</b></dt><dd>Recorded Date</dd>
                    <dt><b>_recordId</b></dt><dd>Response ID</dd>
                    <dt><b>locationLatitude</b></dt><dd>Latitude</dd>
                    <dt><b>locationLongitude</b></dt><dd>Longitude</dd>
                    <dt><b>recipientLastName</b></dt><dd>Recipient Last Name</dd>
            .       <dt><b>recipientFirstName</b></dt><dd>Recipient First Name</dd>
            .       <dt><b>recipientEmail</b></dt><dd>Recipient Email</dd>
            .       <dt><b>externalReference</b></dt><dd>External Reference</dd>
            .       <dt><b>distributionChannel</b></dt><dd>Distribution Channel</dd>
            </dl>
        continuation_token (Union[Unset, str]): Provide the continuation token returned from a previous export in order
            to get new responses recorded (or updated, if `sortByLastModifiedDate` is set to `true`) since that export. Note
            that a token is only returned if either `allowContinuation` or `continuationToken` is set. Continuation tokens
            expire after a week. If you provide a continuation token, it implies `allowContinuation` and you must not
            include the `allowContinuation` field. Example:
            UQhcCBAIGwgGCFkIEBscGhIcHxkZHxwGCEQIEBwYGxoaGhoaGgYITwgQGxwaExgcGhkYE1c.
        allow_continuation (Union[Unset, bool]): Set this to `true` to request a continuation token when this export has
            finished. Note, when this is set to true, you cannot include a filter. Default: True. Example: false.
        include_label_columns (Union[Unset, bool]): For columns that have answer labels, export two columns: one that
            uses recode values and one that uses labels. The label column has an `IsLabelsColumn` field in the 3rd header
            row. Note that this cannot be used with `useLabels`. Use this setting if you wish to export both column labels
            and numeric values in the same export. Default: True. Example: false.
        sort_by_last_modified_date (Union[Unset, bool]): Sort responses by modified date, which represents the date when
            a response was last updated or the creation time if the response hasn't changed since it was recorded. Note,
            when this is set to `true`, `startDate` and/or `endDate`, if provided, will be used to filter responses based on
            the modified date instead of the creation time alone. This parameter cannot be used together with the `filterId`
            parameter. Also, the modified date will be automatically included under an additional `LastModifiedDate` column
            in the export. `sortByLastModifiedDate` can be used in conjunction with `continuationToken` to periodically get
            all the new and modified responses since the last export and is the recommended way of handling that use case.
            Default: True. Example: false.
    """

    format_: ExportCreationRequestFormat = ExportCreationRequestFormat.CSV
    breakout_sets: Union[Unset, bool] = True
    compress: Union[Unset, bool] = True
    end_date: Union[Unset, datetime.datetime] = isoparse("2100-01-01T01:00:00Z")
    export_responses_in_progress: Union[Unset, bool] = True
    filter_id: Union[Unset, str] = UNSET
    format_decimal_as_comma: Union[Unset, bool] = True
    include_display_order: Union[Unset, bool] = True
    limit: Union[Unset, int] = UNSET
    multiselect_seen_unanswered_recode: Union[Unset, int] = UNSET
    newline_replacement: Union[Unset, str] = UNSET
    seen_unanswered_recode: Union[Unset, int] = UNSET
    start_date: Union[Unset, datetime.datetime] = isoparse("1970-01-01T01:00:00Z")
    time_zone: Union[Unset, str] = "UTC"
    use_labels: Union[Unset, bool] = True
    embedded_data_ids: Union[Unset, List[str]] = UNSET
    question_ids: Union[Unset, List[str]] = UNSET
    survey_metadata_ids: Union[Unset, List[str]] = UNSET
    continuation_token: Union[Unset, str] = UNSET
    allow_continuation: Union[Unset, bool] = True
    include_label_columns: Union[Unset, bool] = True
    sort_by_last_modified_date: Union[Unset, bool] = True
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        format_ = self.format_.value

        breakout_sets = self.breakout_sets
        compress = self.compress
        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        export_responses_in_progress = self.export_responses_in_progress
        filter_id = self.filter_id
        format_decimal_as_comma = self.format_decimal_as_comma
        include_display_order = self.include_display_order
        limit = self.limit
        multiselect_seen_unanswered_recode = self.multiselect_seen_unanswered_recode
        newline_replacement = self.newline_replacement
        seen_unanswered_recode = self.seen_unanswered_recode
        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        time_zone = self.time_zone
        use_labels = self.use_labels
        embedded_data_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.embedded_data_ids, Unset):
            embedded_data_ids = self.embedded_data_ids

        question_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.question_ids, Unset):
            question_ids = self.question_ids

        survey_metadata_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.survey_metadata_ids, Unset):
            survey_metadata_ids = self.survey_metadata_ids

        continuation_token = self.continuation_token
        allow_continuation = self.allow_continuation
        include_label_columns = self.include_label_columns
        sort_by_last_modified_date = self.sort_by_last_modified_date

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "format": format_,
            }
        )
        if breakout_sets is not UNSET:
            field_dict["breakoutSets"] = breakout_sets
        if compress is not UNSET:
            field_dict["compress"] = compress
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if export_responses_in_progress is not UNSET:
            field_dict["exportResponsesInProgress"] = export_responses_in_progress
        if filter_id is not UNSET:
            field_dict["filterId"] = filter_id
        if format_decimal_as_comma is not UNSET:
            field_dict["formatDecimalAsComma"] = format_decimal_as_comma
        if include_display_order is not UNSET:
            field_dict["includeDisplayOrder"] = include_display_order
        if limit is not UNSET:
            field_dict["limit"] = limit
        if multiselect_seen_unanswered_recode is not UNSET:
            field_dict[
                "multiselectSeenUnansweredRecode"
            ] = multiselect_seen_unanswered_recode
        if newline_replacement is not UNSET:
            field_dict["newlineReplacement"] = newline_replacement
        if seen_unanswered_recode is not UNSET:
            field_dict["seenUnansweredRecode"] = seen_unanswered_recode
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if time_zone is not UNSET:
            field_dict["timeZone"] = time_zone
        if use_labels is not UNSET:
            field_dict["useLabels"] = use_labels
        if embedded_data_ids is not UNSET:
            field_dict["embeddedDataIds"] = embedded_data_ids
        if question_ids is not UNSET:
            field_dict["questionIds"] = question_ids
        if survey_metadata_ids is not UNSET:
            field_dict["surveyMetadataIds"] = survey_metadata_ids
        if continuation_token is not UNSET:
            field_dict["continuationToken"] = continuation_token
        if allow_continuation is not UNSET:
            field_dict["allowContinuation"] = allow_continuation
        if include_label_columns is not UNSET:
            field_dict["includeLabelColumns"] = include_label_columns
        if sort_by_last_modified_date is not UNSET:
            field_dict["sortByLastModifiedDate"] = sort_by_last_modified_date

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        format_ = ExportCreationRequestFormat(d.pop("format"))

        breakout_sets = d.pop("breakoutSets", UNSET)

        compress = d.pop("compress", UNSET)

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, datetime.datetime]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date)

        export_responses_in_progress = d.pop("exportResponsesInProgress", UNSET)

        filter_id = d.pop("filterId", UNSET)

        format_decimal_as_comma = d.pop("formatDecimalAsComma", UNSET)

        include_display_order = d.pop("includeDisplayOrder", UNSET)

        limit = d.pop("limit", UNSET)

        multiselect_seen_unanswered_recode = d.pop(
            "multiselectSeenUnansweredRecode", UNSET
        )

        newline_replacement = d.pop("newlineReplacement", UNSET)

        seen_unanswered_recode = d.pop("seenUnansweredRecode", UNSET)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.datetime]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date)

        time_zone = d.pop("timeZone", UNSET)

        use_labels = d.pop("useLabels", UNSET)

        embedded_data_ids = cast(List[str], d.pop("embeddedDataIds", UNSET))

        question_ids = cast(List[str], d.pop("questionIds", UNSET))

        survey_metadata_ids = cast(List[str], d.pop("surveyMetadataIds", UNSET))

        continuation_token = d.pop("continuationToken", UNSET)

        allow_continuation = d.pop("allowContinuation", UNSET)

        include_label_columns = d.pop("includeLabelColumns", UNSET)

        sort_by_last_modified_date = d.pop("sortByLastModifiedDate", UNSET)

        export_creation_request = cls(
            format_=format_,
            breakout_sets=breakout_sets,
            compress=compress,
            end_date=end_date,
            export_responses_in_progress=export_responses_in_progress,
            filter_id=filter_id,
            format_decimal_as_comma=format_decimal_as_comma,
            include_display_order=include_display_order,
            limit=limit,
            multiselect_seen_unanswered_recode=multiselect_seen_unanswered_recode,
            newline_replacement=newline_replacement,
            seen_unanswered_recode=seen_unanswered_recode,
            start_date=start_date,
            time_zone=time_zone,
            use_labels=use_labels,
            embedded_data_ids=embedded_data_ids,
            question_ids=question_ids,
            survey_metadata_ids=survey_metadata_ids,
            continuation_token=continuation_token,
            allow_continuation=allow_continuation,
            include_label_columns=include_label_columns,
            sort_by_last_modified_date=sort_by_last_modified_date,
        )

        export_creation_request.additional_properties = d
        return export_creation_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.get_schema_response_result_properties_values_type import (
    GetSchemaResponseResultPropertiesValuesType,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.get_schema_response_result_properties_values_properties import (
        GetSchemaResponseResultPropertiesValuesProperties,
    )


T = TypeVar("T", bound="GetSchemaResponseResultPropertiesValues")


@_attrs_define
class GetSchemaResponseResultPropertiesValues:
    """
    Attributes:
        type (Union[Unset, GetSchemaResponseResultPropertiesValuesType]):
        description (Union[Unset, str]):
        properties (Union[Unset, GetSchemaResponseResultPropertiesValuesProperties]):
        required (Union[Unset, List[str]]): A list of fields that will always be on the survey response. This list is
            the same across all surveys.
    """

    type: Union[Unset, GetSchemaResponseResultPropertiesValuesType] = UNSET
    description: Union[Unset, str] = UNSET
    properties: Union[
        Unset, "GetSchemaResponseResultPropertiesValuesProperties"
    ] = UNSET
    required: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        description = self.description
        properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.properties, Unset):
            properties = self.properties.to_dict()

        required: Union[Unset, List[str]] = UNSET
        if not isinstance(self.required, Unset):
            required = self.required

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if description is not UNSET:
            field_dict["description"] = description
        if properties is not UNSET:
            field_dict["properties"] = properties
        if required is not UNSET:
            field_dict["required"] = required

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.get_schema_response_result_properties_values_properties import (
            GetSchemaResponseResultPropertiesValuesProperties,
        )

        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, GetSchemaResponseResultPropertiesValuesType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = GetSchemaResponseResultPropertiesValuesType(_type)

        description = d.pop("description", UNSET)

        _properties = d.pop("properties", UNSET)
        properties: Union[Unset, GetSchemaResponseResultPropertiesValuesProperties]
        if isinstance(_properties, Unset):
            properties = UNSET
        else:
            properties = GetSchemaResponseResultPropertiesValuesProperties.from_dict(
                _properties
            )

        required = cast(List[str], d.pop("required", UNSET))

        get_schema_response_result_properties_values = cls(
            type=type,
            description=description,
            properties=properties,
            required=required,
        )

        get_schema_response_result_properties_values.additional_properties = d
        return get_schema_response_result_properties_values

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.retrieve_response_response_result_displayed_values import (
        RetrieveResponseResponseResultDisplayedValues,
    )
    from ..models.retrieve_response_response_result_labels import (
        RetrieveResponseResponseResultLabels,
    )
    from ..models.retrieve_response_response_result_values import (
        RetrieveResponseResponseResultValues,
    )


T = TypeVar("T", bound="RetrieveResponseResponseResult")


@_attrs_define
class RetrieveResponseResponseResult:
    """
    Attributes:
        response_id (str): The unique ID for each response
        values (RetrieveResponseResponseResultValues): `values` is a set of key-value pairs corresponding to question-
            answer pairs. There can also be key-value pairs representing meta data for the response such as `startDate` or
            `userLanguage`.
        labels (RetrieveResponseResponseResultLabels): `labels` is a set of key-value pairs where each key corresponds
            to a key in the `values` object and the value is the label of the answer of that question.
        displayed_fields (Union[Unset, List[str]]): All the fields that were displayed during the taking of the survey
        displayed_values (Union[Unset, RetrieveResponseResponseResultDisplayedValues]): For every field, the values that
            were displayed during the taking of the survey
    """

    response_id: str
    values: "RetrieveResponseResponseResultValues"
    labels: "RetrieveResponseResponseResultLabels"
    displayed_fields: Union[Unset, List[str]] = UNSET
    displayed_values: Union[
        Unset, "RetrieveResponseResponseResultDisplayedValues"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        response_id = self.response_id
        values = self.values.to_dict()

        labels = self.labels.to_dict()

        displayed_fields: Union[Unset, List[str]] = UNSET
        if not isinstance(self.displayed_fields, Unset):
            displayed_fields = self.displayed_fields

        displayed_values: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.displayed_values, Unset):
            displayed_values = self.displayed_values.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "responseId": response_id,
                "values": values,
                "labels": labels,
            }
        )
        if displayed_fields is not UNSET:
            field_dict["displayedFields"] = displayed_fields
        if displayed_values is not UNSET:
            field_dict["displayedValues"] = displayed_values

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.retrieve_response_response_result_displayed_values import (
            RetrieveResponseResponseResultDisplayedValues,
        )
        from ..models.retrieve_response_response_result_labels import (
            RetrieveResponseResponseResultLabels,
        )
        from ..models.retrieve_response_response_result_values import (
            RetrieveResponseResponseResultValues,
        )

        d = src_dict.copy()
        response_id = d.pop("responseId")

        values = RetrieveResponseResponseResultValues.from_dict(d.pop("values"))

        labels = RetrieveResponseResponseResultLabels.from_dict(d.pop("labels"))

        displayed_fields = cast(List[str], d.pop("displayedFields", UNSET))

        _displayed_values = d.pop("displayedValues", UNSET)
        displayed_values: Union[Unset, RetrieveResponseResponseResultDisplayedValues]
        if isinstance(_displayed_values, Unset):
            displayed_values = UNSET
        else:
            displayed_values = RetrieveResponseResponseResultDisplayedValues.from_dict(
                _displayed_values
            )

        retrieve_response_response_result = cls(
            response_id=response_id,
            values=values,
            labels=labels,
            displayed_fields=displayed_fields,
            displayed_values=displayed_values,
        )

        retrieve_response_response_result.additional_properties = d
        return retrieve_response_response_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.update_response_request_embedded_data import (
        UpdateResponseRequestEmbeddedData,
    )


T = TypeVar("T", bound="UpdateResponseRequest")


@_attrs_define
class UpdateResponseRequest:
    """
    Attributes:
        survey_id (str): The Survey ID
        embedded_data (UpdateResponseRequestEmbeddedData): `embeddedData` is a JSON object representing the embedded
            data fields to set.
        reset_recorded_date (Union[Unset, bool]): Sets the recorded date to the current time. If false, the recorded
            date will be incremented by one millisecond. Default: True.
    """

    survey_id: str
    embedded_data: "UpdateResponseRequestEmbeddedData"
    reset_recorded_date: Union[Unset, bool] = True
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        survey_id = self.survey_id
        embedded_data = self.embedded_data.to_dict()

        reset_recorded_date = self.reset_recorded_date

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "surveyId": survey_id,
                "embeddedData": embedded_data,
            }
        )
        if reset_recorded_date is not UNSET:
            field_dict["resetRecordedDate"] = reset_recorded_date

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.update_response_request_embedded_data import (
            UpdateResponseRequestEmbeddedData,
        )

        d = src_dict.copy()
        survey_id = d.pop("surveyId")

        embedded_data = UpdateResponseRequestEmbeddedData.from_dict(
            d.pop("embeddedData")
        )

        reset_recorded_date = d.pop("resetRecordedDate", UNSET)

        update_response_request = cls(
            survey_id=survey_id,
            embedded_data=embedded_data,
            reset_recorded_date=reset_recorded_date,
        )

        update_response_request.additional_properties = d
        return update_response_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.get_schema_response_result_schema import GetSchemaResponseResultSchema
from ..models.get_schema_response_result_type import GetSchemaResponseResultType

if TYPE_CHECKING:
    from ..models.get_schema_response_result_properties import (
        GetSchemaResponseResultProperties,
    )


T = TypeVar("T", bound="GetSchemaResponseResult")


@_attrs_define
class GetSchemaResponseResult:
    """
    Attributes:
        schema (GetSchemaResponseResultSchema):
        id (str):
        title (str):
        description (str):
        type (GetSchemaResponseResultType):
        required (List[str]):
        properties (GetSchemaResponseResultProperties):
    """

    schema: GetSchemaResponseResultSchema
    id: str
    title: str
    description: str
    type: GetSchemaResponseResultType
    required: List[str]
    properties: "GetSchemaResponseResultProperties"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        schema = self.schema.value

        id = self.id
        title = self.title
        description = self.description
        type = self.type.value

        required = self.required

        properties = self.properties.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "$schema": schema,
                "$id": id,
                "title": title,
                "description": description,
                "type": type,
                "required": required,
                "properties": properties,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.get_schema_response_result_properties import (
            GetSchemaResponseResultProperties,
        )

        d = src_dict.copy()
        schema = GetSchemaResponseResultSchema(d.pop("$schema"))

        id = d.pop("$id")

        title = d.pop("title")

        description = d.pop("description")

        type = GetSchemaResponseResultType(d.pop("type"))

        required = cast(List[str], d.pop("required"))

        properties = GetSchemaResponseResultProperties.from_dict(d.pop("properties"))

        get_schema_response_result = cls(
            schema=schema,
            id=id,
            title=title,
            description=description,
            type=type,
            required=required,
            properties=properties,
        )

        get_schema_response_result.additional_properties = d
        return get_schema_response_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from qualtrics_utils.codebook.generate import generate_codebook
from qualtrics_utils.survey import Surveys
from qualtrics_utils.sync import sync_sheets, sync_sql
from qualtrics_utils.utils import (
    coalesce_multiselect,
    create_mysql_engine,
    rename_columns,
)

__all__ = [
    "Surveys",
    "generate_codebook",
    "coalesce_multiselect",
    "rename_columns",
    "create_mysql_engine",
    "sync_sheets",
    "sync_sql",
]



import json
from io import BytesIO
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, File, FileJsonType, Unset

if TYPE_CHECKING:
    from ..models.create_response_request import CreateResponseRequest
    from ..models.create_response_with_file_attachments_request_file_mapping import (
        CreateResponseWithFileAttachmentsRequestFileMapping,
    )


T = TypeVar("T", bound="CreateResponseWithFileAttachmentsRequest")


@_attrs_define
class CreateResponseWithFileAttachmentsRequest:
    """
    Attributes:
        response (CreateResponseRequest):
        file_mapping (CreateResponseWithFileAttachmentsRequestFileMapping):
        file1 (Union[Unset, File]):
        file2 (Union[Unset, File]):
        file3 (Union[Unset, File]):
        file4 (Union[Unset, File]):
        file5 (Union[Unset, File]):
    """

    response: "CreateResponseRequest"
    file_mapping: "CreateResponseWithFileAttachmentsRequestFileMapping"
    file1: Union[Unset, File] = UNSET
    file2: Union[Unset, File] = UNSET
    file3: Union[Unset, File] = UNSET
    file4: Union[Unset, File] = UNSET
    file5: Union[Unset, File] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        response = self.response.to_dict()

        file_mapping = self.file_mapping.to_dict()

        file1: Union[Unset, FileJsonType] = UNSET
        if not isinstance(self.file1, Unset):
            file1 = self.file1.to_tuple()

        file2: Union[Unset, FileJsonType] = UNSET
        if not isinstance(self.file2, Unset):
            file2 = self.file2.to_tuple()

        file3: Union[Unset, FileJsonType] = UNSET
        if not isinstance(self.file3, Unset):
            file3 = self.file3.to_tuple()

        file4: Union[Unset, FileJsonType] = UNSET
        if not isinstance(self.file4, Unset):
            file4 = self.file4.to_tuple()

        file5: Union[Unset, FileJsonType] = UNSET
        if not isinstance(self.file5, Unset):
            file5 = self.file5.to_tuple()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "response": response,
                "fileMapping": file_mapping,
            }
        )
        if file1 is not UNSET:
            field_dict["file1"] = file1
        if file2 is not UNSET:
            field_dict["file2"] = file2
        if file3 is not UNSET:
            field_dict["file3"] = file3
        if file4 is not UNSET:
            field_dict["file4"] = file4
        if file5 is not UNSET:
            field_dict["file5"] = file5

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        response = (
            None,
            json.dumps(self.response.to_dict()).encode(),
            "application/json",
        )

        file_mapping = (
            None,
            json.dumps(self.file_mapping.to_dict()).encode(),
            "application/json",
        )

        file1: Union[Unset, FileJsonType] = UNSET
        if not isinstance(self.file1, Unset):
            file1 = self.file1.to_tuple()

        file2: Union[Unset, FileJsonType] = UNSET
        if not isinstance(self.file2, Unset):
            file2 = self.file2.to_tuple()

        file3: Union[Unset, FileJsonType] = UNSET
        if not isinstance(self.file3, Unset):
            file3 = self.file3.to_tuple()

        file4: Union[Unset, FileJsonType] = UNSET
        if not isinstance(self.file4, Unset):
            file4 = self.file4.to_tuple()

        file5: Union[Unset, FileJsonType] = UNSET
        if not isinstance(self.file5, Unset):
            file5 = self.file5.to_tuple()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                key: (None, str(value).encode(), "text/plain")
                for key, value in self.additional_properties.items()
            }
        )
        field_dict.update(
            {
                "response": response,
                "fileMapping": file_mapping,
            }
        )
        if file1 is not UNSET:
            field_dict["file1"] = file1
        if file2 is not UNSET:
            field_dict["file2"] = file2
        if file3 is not UNSET:
            field_dict["file3"] = file3
        if file4 is not UNSET:
            field_dict["file4"] = file4
        if file5 is not UNSET:
            field_dict["file5"] = file5

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.create_response_request import CreateResponseRequest
        from ..models.create_response_with_file_attachments_request_file_mapping import (
            CreateResponseWithFileAttachmentsRequestFileMapping,
        )

        d = src_dict.copy()
        response = CreateResponseRequest.from_dict(d.pop("response"))

        file_mapping = CreateResponseWithFileAttachmentsRequestFileMapping.from_dict(
            d.pop("fileMapping")
        )

        _file1 = d.pop("file1", UNSET)
        file1: Union[Unset, File]
        if isinstance(_file1, Unset):
            file1 = UNSET
        else:
            file1 = File(payload=BytesIO(_file1))

        _file2 = d.pop("file2", UNSET)
        file2: Union[Unset, File]
        if isinstance(_file2, Unset):
            file2 = UNSET
        else:
            file2 = File(payload=BytesIO(_file2))

        _file3 = d.pop("file3", UNSET)
        file3: Union[Unset, File]
        if isinstance(_file3, Unset):
            file3 = UNSET
        else:
            file3 = File(payload=BytesIO(_file3))

        _file4 = d.pop("file4", UNSET)
        file4: Union[Unset, File]
        if isinstance(_file4, Unset):
            file4 = UNSET
        else:
            file4 = File(payload=BytesIO(_file4))

        _file5 = d.pop("file5", UNSET)
        file5: Union[Unset, File]
        if isinstance(_file5, Unset):
            file5 = UNSET
        else:
            file5 = File(payload=BytesIO(_file5))

        create_response_with_file_attachments_request = cls(
            response=response,
            file_mapping=file_mapping,
            file1=file1,
            file2=file2,
            file3=file3,
            file4=file4,
            file5=file5,
        )

        create_response_with_file_attachments_request.additional_properties = d
        return create_response_with_file_attachments_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ExportCreationRequestFormat(str, Enum):
    CSV = "csv"
    JSON = "json"
    NDJSON = "ndjson"
    SPSS = "spss"
    TSV = "tsv"
    XML = "xml"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from io import BytesIO
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.meta import Meta
from ...types import File, Response


def _get_kwargs(
    survey_id: str,
    file_id: str,
) -> Dict[str, Any]:

    pass

    return {
        "method": "get",
        "url": "/surveys/{surveyId}/export-responses/{fileId}/file".format(
            surveyId=survey_id,
            fileId=file_id,
        ),
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[File, Meta]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = File(payload=BytesIO(response.content))

        return response_200
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = Meta.from_dict(response.json())

        return response_404
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[File, Meta]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    survey_id: str,
    file_id: str,
    *,
    client: AuthenticatedClient,
) -> Response[Union[File, Meta]]:
    r""" Get Response Export File

     Retrieves the response export file after the file export process completes. By default, the export
    file is returned as a ZIP file, but you can also request the export file in a different file format
    when you start the export with the file format you specified in [Start Response
    Export](../../../../reference/responseImportsExports.json/paths/~1surveys~1{surveyId}~1export-
    responses/post) call. The supported formats for the export file are CSV, NDJSON, TSV, XML, JSON, or
    SPSS. Without an extension, the export file defaults to a ZIP file.

    The compressed file inside the retrieved ZIP file has this naming convention: {Survey Project
    Name}.{Export Format}. Note that the file does not return time and date of export. For date and time
    information, you need to record this information manually during the export process.

    The example below shows a sample cURL request for this endpoint. Please note that you need to
    include your `surveyId` and the `fileId` in the URL of this request. The `fileId` is returned in the
    response to the [Get Response Export
    Progress](../../../../reference/responseImportsExports.json/paths/~1surveys~1{surveyId}~1export-
    responses~1{exportProgressId}/get) call.

    <!-- title: Retrieve file -->
    ```shell
    curl --request GET \
    --url https://ca1.qualtrics.com/API/v3/surveys/surveyId/export-responses/fileId/file \
    --header 'Content-Type: application/json' \
    --header 'X-API-TOKEN: ' \
    -OJ myResponseFile
    ```

    <!-- theme: warning -->
    ### Try it Feature Limitation
    >The 'Try it' tab above does not allow data export. Please use cURL or the Postman app to send this
    request.

    <!-- theme: info -->
    ### File Expiration
    >Export files expire one week after the completion of an export. After that expiration, the file is
    no longer available, and you will have to start the export request again.

    Args:
        survey_id (str):
        file_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[File, Meta]]
     """

    kwargs = _get_kwargs(
        survey_id=survey_id,
        file_id=file_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    survey_id: str,
    file_id: str,
    *,
    client: AuthenticatedClient,
) -> Optional[Union[File, Meta]]:
    r""" Get Response Export File

     Retrieves the response export file after the file export process completes. By default, the export
    file is returned as a ZIP file, but you can also request the export file in a different file format
    when you start the export with the file format you specified in [Start Response
    Export](../../../../reference/responseImportsExports.json/paths/~1surveys~1{surveyId}~1export-
    responses/post) call. The supported formats for the export file are CSV, NDJSON, TSV, XML, JSON, or
    SPSS. Without an extension, the export file defaults to a ZIP file.

    The compressed file inside the retrieved ZIP file has this naming convention: {Survey Project
    Name}.{Export Format}. Note that the file does not return time and date of export. For date and time
    information, you need to record this information manually during the export process.

    The example below shows a sample cURL request for this endpoint. Please note that you need to
    include your `surveyId` and the `fileId` in the URL of this request. The `fileId` is returned in the
    response to the [Get Response Export
    Progress](../../../../reference/responseImportsExports.json/paths/~1surveys~1{surveyId}~1export-
    responses~1{exportProgressId}/get) call.

    <!-- title: Retrieve file -->
    ```shell
    curl --request GET \
    --url https://ca1.qualtrics.com/API/v3/surveys/surveyId/export-responses/fileId/file \
    --header 'Content-Type: application/json' \
    --header 'X-API-TOKEN: ' \
    -OJ myResponseFile
    ```

    <!-- theme: warning -->
    ### Try it Feature Limitation
    >The 'Try it' tab above does not allow data export. Please use cURL or the Postman app to send this
    request.

    <!-- theme: info -->
    ### File Expiration
    >Export files expire one week after the completion of an export. After that expiration, the file is
    no longer available, and you will have to start the export request again.

    Args:
        survey_id (str):
        file_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[File, Meta]
     """

    return sync_detailed(
        survey_id=survey_id,
        file_id=file_id,
        client=client,
    ).parsed


async def asyncio_detailed(
    survey_id: str,
    file_id: str,
    *,
    client: AuthenticatedClient,
) -> Response[Union[File, Meta]]:
    r""" Get Response Export File

     Retrieves the response export file after the file export process completes. By default, the export
    file is returned as a ZIP file, but you can also request the export file in a different file format
    when you start the export with the file format you specified in [Start Response
    Export](../../../../reference/responseImportsExports.json/paths/~1surveys~1{surveyId}~1export-
    responses/post) call. The supported formats for the export file are CSV, NDJSON, TSV, XML, JSON, or
    SPSS. Without an extension, the export file defaults to a ZIP file.

    The compressed file inside the retrieved ZIP file has this naming convention: {Survey Project
    Name}.{Export Format}. Note that the file does not return time and date of export. For date and time
    information, you need to record this information manually during the export process.

    The example below shows a sample cURL request for this endpoint. Please note that you need to
    include your `surveyId` and the `fileId` in the URL of this request. The `fileId` is returned in the
    response to the [Get Response Export
    Progress](../../../../reference/responseImportsExports.json/paths/~1surveys~1{surveyId}~1export-
    responses~1{exportProgressId}/get) call.

    <!-- title: Retrieve file -->
    ```shell
    curl --request GET \
    --url https://ca1.qualtrics.com/API/v3/surveys/surveyId/export-responses/fileId/file \
    --header 'Content-Type: application/json' \
    --header 'X-API-TOKEN: ' \
    -OJ myResponseFile
    ```

    <!-- theme: warning -->
    ### Try it Feature Limitation
    >The 'Try it' tab above does not allow data export. Please use cURL or the Postman app to send this
    request.

    <!-- theme: info -->
    ### File Expiration
    >Export files expire one week after the completion of an export. After that expiration, the file is
    no longer available, and you will have to start the export request again.

    Args:
        survey_id (str):
        file_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[File, Meta]]
     """

    kwargs = _get_kwargs(
        survey_id=survey_id,
        file_id=file_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    survey_id: str,
    file_id: str,
    *,
    client: AuthenticatedClient,
) -> Optional[Union[File, Meta]]:
    r""" Get Response Export File

     Retrieves the response export file after the file export process completes. By default, the export
    file is returned as a ZIP file, but you can also request the export file in a different file format
    when you start the export with the file format you specified in [Start Response
    Export](../../../../reference/responseImportsExports.json/paths/~1surveys~1{surveyId}~1export-
    responses/post) call. The supported formats for the export file are CSV, NDJSON, TSV, XML, JSON, or
    SPSS. Without an extension, the export file defaults to a ZIP file.

    The compressed file inside the retrieved ZIP file has this naming convention: {Survey Project
    Name}.{Export Format}. Note that the file does not return time and date of export. For date and time
    information, you need to record this information manually during the export process.

    The example below shows a sample cURL request for this endpoint. Please note that you need to
    include your `surveyId` and the `fileId` in the URL of this request. The `fileId` is returned in the
    response to the [Get Response Export
    Progress](../../../../reference/responseImportsExports.json/paths/~1surveys~1{surveyId}~1export-
    responses~1{exportProgressId}/get) call.

    <!-- title: Retrieve file -->
    ```shell
    curl --request GET \
    --url https://ca1.qualtrics.com/API/v3/surveys/surveyId/export-responses/fileId/file \
    --header 'Content-Type: application/json' \
    --header 'X-API-TOKEN: ' \
    -OJ myResponseFile
    ```

    <!-- theme: warning -->
    ### Try it Feature Limitation
    >The 'Try it' tab above does not allow data export. Please use cURL or the Postman app to send this
    request.

    <!-- theme: info -->
    ### File Expiration
    >Export files expire one week after the completion of an export. After that expiration, the file is
    no longer available, and you will have to start the export request again.

    Args:
        survey_id (str):
        file_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[File, Meta]
     """

    return (
        await asyncio_detailed(
            survey_id=survey_id,
            file_id=file_id,
            client=client,
        )
    ).parsed



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.import_status_response import ImportStatusResponse
from ...models.import_status_response_not_found import ImportStatusResponseNotFound
from ...models.internal_error import InternalError
from ...types import Response, Unset


def _get_kwargs(
    survey_id: str,
    import_progress_id: str,
    *,
    idempotency_key: Union[Unset, str] = "",
) -> Dict[str, Any]:
    headers = {}
    if not isinstance(idempotency_key, Unset):
        headers["Idempotency-Key"] = idempotency_key

    return {
        "method": "get",
        "url": "/surveys/{surveyId}/import-responses/{importProgressId}".format(
            surveyId=survey_id,
            importProgressId=import_progress_id,
        ),
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[ImportStatusResponse, ImportStatusResponseNotFound, InternalError]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = ImportStatusResponse.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = ImportStatusResponseNotFound.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = InternalError.from_dict(response.json())

        return response_500
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[ImportStatusResponse, ImportStatusResponseNotFound, InternalError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    survey_id: str,
    import_progress_id: str,
    *,
    client: AuthenticatedClient,
    idempotency_key: Union[Unset, str] = "",
) -> Response[Union[ImportStatusResponse, ImportStatusResponseNotFound, InternalError]]:
    """Get Import Progress

     Retrieves the status of an import. See the [Response Import/Export API
    Overview](responseImportsExports.json) for more detail on how to use this endpoint within an import
    workflow.

    If you receive a status of failed, record your requestId and try your import again. Contact support
    with your requestId and date and time of the call to troubleshoot.



    Args:
        survey_id (str):
        import_progress_id (str):
        idempotency_key (Union[Unset, str]):  Default: ''.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ImportStatusResponse, ImportStatusResponseNotFound, InternalError]]
    """

    kwargs = _get_kwargs(
        survey_id=survey_id,
        import_progress_id=import_progress_id,
        idempotency_key=idempotency_key,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    survey_id: str,
    import_progress_id: str,
    *,
    client: AuthenticatedClient,
    idempotency_key: Union[Unset, str] = "",
) -> Optional[Union[ImportStatusResponse, ImportStatusResponseNotFound, InternalError]]:
    """Get Import Progress

     Retrieves the status of an import. See the [Response Import/Export API
    Overview](responseImportsExports.json) for more detail on how to use this endpoint within an import
    workflow.

    If you receive a status of failed, record your requestId and try your import again. Contact support
    with your requestId and date and time of the call to troubleshoot.



    Args:
        survey_id (str):
        import_progress_id (str):
        idempotency_key (Union[Unset, str]):  Default: ''.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ImportStatusResponse, ImportStatusResponseNotFound, InternalError]
    """

    return sync_detailed(
        survey_id=survey_id,
        import_progress_id=import_progress_id,
        client=client,
        idempotency_key=idempotency_key,
    ).parsed


async def asyncio_detailed(
    survey_id: str,
    import_progress_id: str,
    *,
    client: AuthenticatedClient,
    idempotency_key: Union[Unset, str] = "",
) -> Response[Union[ImportStatusResponse, ImportStatusResponseNotFound, InternalError]]:
    """Get Import Progress

     Retrieves the status of an import. See the [Response Import/Export API
    Overview](responseImportsExports.json) for more detail on how to use this endpoint within an import
    workflow.

    If you receive a status of failed, record your requestId and try your import again. Contact support
    with your requestId and date and time of the call to troubleshoot.



    Args:
        survey_id (str):
        import_progress_id (str):
        idempotency_key (Union[Unset, str]):  Default: ''.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ImportStatusResponse, ImportStatusResponseNotFound, InternalError]]
    """

    kwargs = _get_kwargs(
        survey_id=survey_id,
        import_progress_id=import_progress_id,
        idempotency_key=idempotency_key,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    survey_id: str,
    import_progress_id: str,
    *,
    client: AuthenticatedClient,
    idempotency_key: Union[Unset, str] = "",
) -> Optional[Union[ImportStatusResponse, ImportStatusResponseNotFound, InternalError]]:
    """Get Import Progress

     Retrieves the status of an import. See the [Response Import/Export API
    Overview](responseImportsExports.json) for more detail on how to use this endpoint within an import
    workflow.

    If you receive a status of failed, record your requestId and try your import again. Contact support
    with your requestId and date and time of the call to troubleshoot.



    Args:
        survey_id (str):
        import_progress_id (str):
        idempotency_key (Union[Unset, str]):  Default: ''.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ImportStatusResponse, ImportStatusResponseNotFound, InternalError]
    """

    return (
        await asyncio_detailed(
            survey_id=survey_id,
            import_progress_id=import_progress_id,
            client=client,
            idempotency_key=idempotency_key,
        )
    ).parsed



from enum import Enum


class SchemaPropertyType(str, Enum):
    ARRAY = "array"
    INTEGER = "integer"
    NUMBER = "number"
    OBJECT = "object"
    STRING = "string"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.get_schema_response_result_properties_displayed_values_type import (
    GetSchemaResponseResultPropertiesDisplayedValuesType,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.get_schema_response_result_properties_displayed_values_additional_properties import (
        GetSchemaResponseResultPropertiesDisplayedValuesAdditionalProperties,
    )


T = TypeVar("T", bound="GetSchemaResponseResultPropertiesDisplayedValues")


@_attrs_define
class GetSchemaResponseResultPropertiesDisplayedValues:
    """
    Attributes:
        description (Union[Unset, str]):
        type (Union[Unset, GetSchemaResponseResultPropertiesDisplayedValuesType]):
        additional_properties (Union[Unset, GetSchemaResponseResultPropertiesDisplayedValuesAdditionalProperties]):
    """

    description: Union[Unset, str] = UNSET
    type: Union[Unset, GetSchemaResponseResultPropertiesDisplayedValuesType] = UNSET
    additional_properties: Union[
        Unset, "GetSchemaResponseResultPropertiesDisplayedValuesAdditionalProperties"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        description = self.description
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        additional_properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.additional_properties, Unset):
            additional_properties = self.additional_properties.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if description is not UNSET:
            field_dict["description"] = description
        if type is not UNSET:
            field_dict["type"] = type
        if additional_properties is not UNSET:
            field_dict["additionalProperties"] = additional_properties

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.get_schema_response_result_properties_displayed_values_additional_properties import (
            GetSchemaResponseResultPropertiesDisplayedValuesAdditionalProperties,
        )

        d = src_dict.copy()
        description = d.pop("description", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, GetSchemaResponseResultPropertiesDisplayedValuesType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = GetSchemaResponseResultPropertiesDisplayedValuesType(_type)

        _additional_properties = d.pop("additionalProperties", UNSET)
        additional_properties: Union[
            Unset, GetSchemaResponseResultPropertiesDisplayedValuesAdditionalProperties
        ]
        if isinstance(_additional_properties, Unset):
            additional_properties = UNSET
        else:
            additional_properties = GetSchemaResponseResultPropertiesDisplayedValuesAdditionalProperties.from_dict(
                _additional_properties
            )

        get_schema_response_result_properties_displayed_values = cls(
            description=description,
            type=type,
            additional_properties=additional_properties,
        )

        get_schema_response_result_properties_displayed_values.additional_properties = d
        return get_schema_response_result_properties_displayed_values

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
